@@ -214,12 +214,6 @@ eOSState cPilotskinOsd::ProcessKey(eKeys Key)
   if (state == osUnknown) {
     switch (Key) {
       case kUser1 ... kUser9:
-	 if ((NORMALKEY(Key) == kUser1 && config.red==1)
-	   || (NORMALKEY(Key) == kUser2 && config.red==2)
-	    || (NORMALKEY(Key) == kUser3 && config.red==3)
-	     || (NORMALKEY(Key) == kUser4 && config.red==4)
-	      || (NORMALKEY(Key) == kUser5 && config.red==5)
-	       || (NORMALKEY(Key) == kUser6 && config.red==6))
 	 {
            return osEnd;
          }
@@ -375,23 +369,26 @@ eOSState cPilotskinOsd::ProcessKey(eKeys Key)
       }
       break;
       case kGreen: {
-	 if (extraInfo == false)  {
-           // Display extra infos
-           extraInfo = true;
-           offset = 0;
-	   type = 2; // PRESENT
-           DisplayExtraInfo();
-           return osContinue;
-         } else {
-           // Display normal info
-           extraInfo = false;
-	   DisplayBitmap();
-           DrawMenu(0,0);
-           DisplayBitmap();
-           return osContinue;
-	 }
-      }
-      break;
+	 if (Present)
+	  {
+	    if (extraInfo == false)  {
+                // Display extra infos
+                extraInfo = true;
+                offset = 0;
+	        type = 2; // PRESENT
+                DisplayExtraInfo();
+                return osContinue;
+              } else {
+                // Display normal info
+                extraInfo = false;
+	        DisplayBitmap();
+                DrawMenu(0,0);
+                DisplayBitmap();
+                return osContinue;
+	      }
+	  }
+       }
+       break;
       case kYellow|k_Repeat:
       case kYellow: {
 	 if (extraInfo == false) {
