@@ -6,9 +6,6 @@
 class cPilotskinSetup : public cMenuSetupPage 
 {
 private:
-  int Red;
-  int Green;
-  int Blue;
   int CloseOnSwitch;
   int Hidemenu;
 protected:
