@@ -26,13 +26,17 @@
 #include <dlfcn.h>
 
 #include <linux/dvb/ca.h>
+#include <vdr/channels.h>
 #include <vdr/ci.h>
 #include <vdr/dvbdevice.h>
+#ifndef SASC
+#if APIVERSNUM >= 10500
 #include <vdr/dvbci.h>
-#include <vdr/channels.h>
+#endif
 #include <vdr/thread.h>
 
 #include "FFdecsa/FFdecsa.h"
+#endif //SASC
 
 #include "cam.h"
 #include "scsetup.h"
@@ -343,7 +347,7 @@ bool cLogChain::Parse(const unsigned char *cat)
 class cLogger : public cAction {
 private:
   int cardNum;
-  bool softCSA, up;
+  bool softCSA, up, prescan;
   cSimpleList<cLogChain> chains;
   cSimpleList<cEcmInfo> active;
   //
@@ -351,7 +355,7 @@ private:
   int catVers;
   //
   cPidFilter *AddFilter(int Pid, int Section, int Mask, int Mode, int IdleTime, bool Crc);
-  void SetChains(void);
+  void SetChains(bool forcestop=false);
   void ClearChains(void);
   void StartChain(cLogChain *chain);
   void StopChain(cLogChain *chain, bool force);
@@ -363,13 +367,14 @@ public:
   void EcmStatus(const cEcmInfo *ecm, bool on);
   void Up(void);
   void Down(void);
+  void PreScan(void);
   };
 
 cLogger::cLogger(int CardNum, bool soft)
 :cAction("logger",CardNum)
 {
   cardNum=CardNum; softCSA=soft;
-  catfilt=0; up=false;
+  catfilt=0; up=prescan=false;
   Priority(10);
 }
 
@@ -397,11 +402,18 @@ void cLogger::Down(void)
     PRINTF(L_CORE_AUEXTRA,"%d: DOWN",cardNum);
     ClearChains();
     DelAllFilter();
-    catfilt=0; up=false;
+    catfilt=0; up=prescan=false;
     }
   Unlock();
 }
 
+void cLogger::PreScan(void)
+{
+  Lock();
+  prescan=true; Up(); SetChains();
+  Unlock();
+}
+
 void cLogger::EcmStatus(const cEcmInfo *ecm, bool on)
 {
   Lock();
@@ -419,15 +431,16 @@ void cLogger::EcmStatus(const cEcmInfo *ecm, bool on)
         break;
         }
     }
-  SetChains();
+  bool presave=prescan; prescan=false;
+  SetChains(presave);
   Unlock();
 }
 
-void cLogger::SetChains(void)
+void cLogger::SetChains(bool forcestop)
 {
   for(cLogChain *chain=chains.First(); chain; chain=chains.Next(chain)) {
     bool act=false;
-    if(ScSetup.AutoUpdate>1) act=true;
+    if(ScSetup.AutoUpdate>1 || prescan) act=true;
     else if(ScSetup.AutoUpdate==1) {
       for(cEcmInfo *e=active.First(); e; e=active.Next(e))
         if((e->emmCaId && chain->caid==e->emmCaId) || chain->caid==e->caId) {
@@ -435,7 +448,7 @@ void cLogger::SetChains(void)
           }
        }
     if(act) StartChain(chain);
-    else StopChain(chain,false);
+    else StopChain(chain,forcestop);
     }
 }
 
@@ -719,46 +732,6 @@ bool cEcmCache::ParseLine(const char *line, bool fromCache)
   return res;
 }
 
-// -- cPrgPid ---------------------------------------------------------------------
-
-class cPrgPid : public cSimpleItem {
-private:
-  int type;
-  int pid;
-  bool proc;
-public:
-  cPrgPid(int Type, int Pid);
-  int Pid(void) { return pid; }
-  int Type(void) { return type; }
-  bool Proc(void) { return proc; }
-  void Proc(bool is) { proc=is; };
-  };
-
-cPrgPid::cPrgPid(int Type, int Pid)
-{
-  type=Type; pid=Pid;
-  proc=false;
-}
-
-// -- cPrg ---------------------------------------------------------------------
-
-class cPrg : public cSimpleItem {
-private:
-  int prg;
-  bool isUpdate;
-public:
-  cSimpleList<cPrgPid> pids;
-  //
-  cPrg(int Prg, bool IsUpdate);
-  int Prg(void) { return prg; }
-  bool IsUpdate(void) { return isUpdate; }
-  };
-
-cPrg::cPrg(int Prg, bool IsUpdate)
-{
-  prg=Prg; isUpdate=IsUpdate;
-}
-
 // -- cEcmSys ------------------------------------------------------------------
 
 class cEcmPri : public cSimpleItem {
@@ -1283,9 +1256,9 @@ void cEcmHandler::EcmFail(void)
 void cEcmHandler::ParseCAInfo(int SysId)
 {
   unsigned char buff[2048];
-  int casys[MAXCAIDS+1];
+  caid_t casys[MAXCAIDS+1];
   if(SysId==0xFFFF) {
-    if(!cam->GetPrgCaids(sid,casys)) {
+    if(!cam->GetPrgCaids(filterSource,filterTransponder,filterSid,casys)) {
       PRINTF(L_CORE_ECM,"%s: no CAIDs for SID %d",id,sid);
       return;
       }
@@ -1295,7 +1268,7 @@ void cEcmHandler::ParseCAInfo(int SysId)
     casys[1]=0;
     }
   bool streamFlag;
-  int len=GetCaDescriptors(filterSource,filterTransponder,sid,casys,sizeof(buff),buff,streamFlag);
+  int len=GetCaDescriptors(filterSource,filterTransponder,filterSid,casys,sizeof(buff),buff,streamFlag);
   if(len>0) {
     if(dolog) PRINTF(L_CORE_ECM,"%s: got CaDescriptors for SID %d (len=%d)",id,sid,len);
     HEXDUMP(L_HEX_PMT,buff,len,"PMT");
@@ -1391,6 +1364,14 @@ void cCam::Tune(const cChannel *channel)
   else PRINTF(L_CORE_PIDS,"%d: tune to same source/transponder",cardNum);
 }
 
+void cCam::PostTune(void)
+{
+  if(true) { //XXX configurable??
+    LogStartup();
+    if(logger) logger->PreScan();
+    }
+}
+
 void cCam::SetPid(int type, int pid, bool on)
 {
   cMutexLock lock(this);
@@ -1422,7 +1403,7 @@ void cCam::AddPrg(cPrg *prg)
       break;
       }
   bool needZero=!IsSoftCSA() && (islive || !ScSetup.ConcurrentFF);
-  bool noshift=IsSoftCSA() || prg->IsUpdate();
+  bool noshift=IsSoftCSA() || (prg->IsUpdate() && prg->pids.Count()==0);
   PRINTF(L_CORE_PIDS,"%d: %s SID %d (zero=%d noshift=%d)",cardNum,prg->IsUpdate()?"update":"add",prg->Prg(),needZero,noshift);
   if(prg->pids.Count()>0) {
     LBSTART(L_CORE_PIDS);
@@ -1438,6 +1419,15 @@ void cCam::AddPrg(cPrg *prg)
     }
 }
 
+bool cCam::HasPrg(int prg)
+{
+  cMutexLock lock(this);
+  for(cEcmHandler *handler=handlerList.First(); handler; handler=handlerList.Next(handler))
+    if(!handler->IsIdle() && handler->Sid()==prg)
+      return true;
+  return false;
+}
+
 char *cCam::CurrentKeyStr(int num)
 {
   cMutexLock lock(this);
@@ -1452,7 +1442,7 @@ bool cCam::Active(void)
   cMutexLock lock(this);
   for(cEcmHandler *handler=handlerList.First(); handler; handler=handlerList.Next(handler))
     if(!handler->IsIdle()) {
-      PRINTF(L_GEN_INFO,"Handler %s on card %d is non-idle",handler->Id(),cardNum);
+      PRINTF(L_GEN_INFO,"handler %s on card %d is not idle",handler->Id(),cardNum);
       return true;
       }
   return false;
@@ -1466,24 +1456,32 @@ void cCam::HouseKeeping(void)
     if(handler->IsRemoveable()) RemHandler(handler);
     handler=next;
     }
+/* XXX
   if(handlerList.Count()<1) {
     delete hookman; hookman=0;
     delete logger; logger=0;
     }
+*/
 }
 
-bool cCam::GetPrgCaids(int prg, int *c)
+bool cCam::GetPrgCaids(int source, int transponder, int prg, caid_t *c)
 {
-  return device->GetPrgCaids(prg,c);
+  return device->GetPrgCaids(source,transponder,prg,c);
+}
+
+void cCam::LogStartup(void)
+{
+  cMutexLock lock(this);
+  if(!logger && ScSetup.AutoUpdate) {
+    logger=new cLogger(cardNum,IsSoftCSA());
+    LogStatsUp();
+    }
 }
 
 void cCam::LogEcmStatus(const cEcmInfo *ecm, bool on)
 {
   cMutexLock lock(this);
-  if(!logger && on && ScSetup.AutoUpdate) {
-    logger=new cLogger(cardNum,IsSoftCSA());
-    LogStatsUp();
-    }
+  if(on) LogStartup();
   if(logger) logger->EcmStatus(ecm,on);
 }
 
@@ -1527,7 +1525,7 @@ void cCam::WriteCW(int index, unsigned char *cw, bool force)
       memcpy(&last[0],&cw[0],8);
       ca_descr.parity=0;
       memcpy(ca_descr.cw,&cw[0],8);
-      if(!device->SetCaDescr(&ca_descr))
+      if(!device->SetCaDescr(&ca_descr,force))
         PRINTF(L_GEN_ERROR,"CA_SET_DESCR failed (%s). Expect a black screen.",strerror(errno));
       }
 
@@ -1535,7 +1533,7 @@ void cCam::WriteCW(int index, unsigned char *cw, bool force)
       memcpy(&last[8],&cw[8],8);
       ca_descr.parity=1;
       memcpy(ca_descr.cw,&cw[8],8);
-      if(!device->SetCaDescr(&ca_descr))
+      if(!device->SetCaDescr(&ca_descr,force))
         PRINTF(L_GEN_ERROR,"CA_SET_DESCR failed (%s). Expect a black screen.",strerror(errno));
       }
     }
@@ -1664,23 +1662,27 @@ void cCam::RemHandler(cEcmHandler *handler)
   indexMap[idx]=0;
 }
 
+#ifndef SASC
+
 // --- cChannelCaids -----------------------------------------------------------
 
+#if APIVERSNUM >= 10500
+
 class cChannelCaids : public cSimpleItem {
 private:
   int prg, source, transponder;
   int numcaids;
-  int caids[MAX_CI_SLOT_CAIDS+1];
+  caid_t caids[MAX_CI_SLOT_CAIDS+1];
 public:
   cChannelCaids(cChannel *channel);
   bool IsChannel(cChannel *channel);
   void Sort(void);
-  void Del(int caid);
-  bool HasCaid(int caid);
+  void Del(caid_t caid);
+  bool HasCaid(caid_t caid);
   bool Same(cChannelCaids *ch);
   void HistAdd(unsigned short *hist);
   void Dump(int n);
-  const int *Caids(void) { caids[numcaids]=0; return caids; }
+  const caid_t *Caids(void) { caids[numcaids]=0; return caids; }
   int NumCaids(void) { return numcaids; }
   };
 
@@ -1688,7 +1690,7 @@ cChannelCaids::cChannelCaids(cChannel *channel)
 {
   prg=channel->Sid(); source=channel->Source(); transponder=channel->Transponder();
   numcaids=0;
-  for(const int *ids=channel->Caids(); *ids; ids++)
+  for(const caid_t *ids=channel->Caids(); *ids; ids++)
     if(numcaids<MAX_CI_SLOT_CAIDS) caids[numcaids++]=*ids;
   Sort();
 }
@@ -1700,7 +1702,7 @@ bool cChannelCaids::IsChannel(cChannel *channel)
 
 void cChannelCaids::Sort(void)
 {
-  int tmp[MAX_CI_SLOT_CAIDS];
+  caid_t tmp[MAX_CI_SLOT_CAIDS];
   int c=0xFFFF;
   for(int i=0; i<numcaids; i++) {
     int d=0;
@@ -1710,7 +1712,7 @@ void cChannelCaids::Sort(void)
   memcpy(caids,tmp,sizeof(caids));
 }
 
-void cChannelCaids::Del(int caid)
+void cChannelCaids::Del(caid_t caid)
 {
   for(int i=0; i<numcaids; i++)
     if(caids[i]==caid) {
@@ -1721,7 +1723,7 @@ void cChannelCaids::Del(int caid)
       }
 }
 
-bool cChannelCaids::HasCaid(int caid)
+bool cChannelCaids::HasCaid(caid_t caid)
 {
   for(int i=0; i<numcaids; i++) if(caids[i]==caid) return true;
   return false;
@@ -1730,7 +1732,7 @@ bool cChannelCaids::HasCaid(int caid)
 bool cChannelCaids::Same(cChannelCaids *ch)
 {
   if(numcaids!=ch->numcaids) return false;
-  return memcmp(caids,ch->caids,numcaids*sizeof(int))==0;
+  return memcmp(caids,ch->caids,numcaids*sizeof(caid_t))==0;
 }
 
 void cChannelCaids::HistAdd(unsigned short *hist)
@@ -1742,7 +1744,7 @@ void cChannelCaids::Dump(int n)
 {
   LBSTART(L_CORE_CAIDS);
   LBPUT("%d: channel %d/%x/%x",n,prg,source,transponder);
-  for(const int *ids=Caids(); *ids; ids++) LBPUT(" %04x",*ids);
+  for(const caid_t *ids=Caids(); *ids; ids++) LBPUT(" %04x",*ids);
   LBEND();
 }
 
@@ -1767,7 +1769,7 @@ cChannelList::cChannelList(int N)
 void cChannelList::CheckIgnore(void)
 {
   for(cChannelCaids *ch=First(); ch; ch=Next(ch)) {
-    const int *ids=ch->Caids();
+    const caid_t *ids=ch->Caids();
     while(*ids) {
       int pri=0;
       if(!cSystems::FindIdentBySysId(*ids,false,pri)) {
@@ -1853,7 +1855,7 @@ private:
   //
   cTimeMs caidTimer;
   int version[MAX_CI_SLOTS];
-  int caids[MAX_CI_SLOTS][MAX_CI_SLOT_CAIDS+1];
+  caid_t caids[MAX_CI_SLOTS][MAX_CI_SLOT_CAIDS+1];
   int tcid;
   //
   cTimeMs readTimer, writeTimer;
@@ -1901,24 +1903,26 @@ public:
 
 cScCamSlot::cScCamSlot(cScCiAdapter *ca, int CardIndex, int Slot)
 :cCamSlot(ca)
-,cRingBufferLinear(KILOBYTE(2),5+1)
+,cRingBufferLinear(KILOBYTE(2),5+1,false,"SC-CI slot answer")
+,checkTimer(-SLOT_CAID_CHECK-1000)
 {
   ciadapter=ca; cardIndex=CardIndex; slot=Slot;
-  checkTimer.Set(-SLOT_CAID_CHECK-1000);
   version=0; caids[0]=0; doReply=false; lastStatus=msReset;
   Reset(false);
 }
 
 eModuleStatus cScCamSlot::Status(void)
 {
-  if(lastStatus!=msReady) Check();
   eModuleStatus status;
   if(reset) { 
     status=msReset;
     if(resetTimer.TimedOut()) reset=false;
     }
   else if(caids[0]) status=msReady;
-  else status=msNone;
+  else {
+    status=msPresent; //msNone;
+    Check();
+    }
   if(status!=lastStatus) {
     static const char *stext[] = { "none","reset","present","ready" };
     PRINTF(L_CORE_CI,"%d.%d: status '%s'",cardIndex,slot,stext[status]);
@@ -2076,7 +2080,7 @@ cScCiAdapter::cScCiAdapter(cDevice *Device, int CardIndex, cCam *Cam)
   memset(version,0,sizeof(version));
   memset(slots,0,sizeof(slots));
   SetDescription("SC-CI adapter on device %d",cardIndex);
-  rb=new cRingBufferLinear(KILOBYTE(5),6+1);
+  rb=new cRingBufferLinear(KILOBYTE(5),6+1,false,"SC-CI adapter read");
   if(rb) {
     rb->SetTimeouts(0,CAM_READ_TIMEOUT);
 /*
@@ -2148,7 +2152,8 @@ void cScCiAdapter::BuildCaids(bool force)
     list.CheckIgnore();
     list.Unique();
 
-    int n=0, h, c[MAX_CI_SLOT_CAIDS+1];
+    int n=0, h;
+    caid_t c[MAX_CI_SLOT_CAIDS+1];
     memset(c,0,sizeof(c));
     do {
       if((h=list.Histo())<0) break;
@@ -2166,17 +2171,15 @@ void cScCiAdapter::BuildCaids(bool force)
     if((version[0]==0 && c[0]!=0) || memcmp(caids[0],c,sizeof(caids[0]))) {
       memcpy(caids[0],c,sizeof(caids[0]));
       version[0]++;
+      if(version[0]>0) {
+        LBSTART(L_CORE_CI);
+        LBPUT("card %d, slot %d (v=%2d) caids:",cardIndex,0,version[0]);
+        for(int i=0; caids[0][i]; i++) LBPUT(" %04x",caids[0][i]);
+        LBEND();
+        }
       }
     ciMutex.Unlock();
 
-    LBSTART(L_CORE_CI);
-    for(int j=0; j<MAX_CI_SLOTS; j++) {
-      if(version[j]>0) {
-        LBPUT("card %d, slot %d (v=%2d) caids:",cardIndex,j,version[j]); for(int i=0; caids[j][i]; i++) LBPUT(" %04x",caids[j][i]);
-        LBFLUSH();
-        }
-      }
-    LBEND();
     caidTimer.Set(CAID_TIME);
     }
 }
@@ -2276,6 +2279,7 @@ void cScCiAdapter::Write(const unsigned char *buff, int len)
 bool cScCiAdapter::Reset(int Slot)
 {
   cMutexLock lock(&ciMutex);
+  PRINTF(L_CORE_CI,"%d: reset of slot %d requested",cardIndex,Slot);
   return slots[Slot] ? slots[Slot]->Reset():false;
 }
 
@@ -2292,33 +2296,45 @@ bool cScCiAdapter::Assign(cDevice *Device, bool Query)
   return Device ? (Device==device) : true;
 }
 
+#endif //APIVERSNUM >= 10500
+
 // -- cDeCSA -------------------------------------------------------------------
 
 #define MAX_CSA_PIDS 8192
 #define MAX_CSA_IDX  16
 
-class cDeCSA : public cMutex {
+//#define DEBUG_CSA
+
+class cDeCSA {
 private:
   int cs;
   unsigned char **range;
   unsigned char pidmap[MAX_CSA_PIDS];
   void *keys[MAX_CSA_IDX];
+  unsigned int even_odd[MAX_CSA_IDX];
+  cMutex mutex;
+  cCondVar wait;
+  bool active;
+  int cardindex;
   //
   bool GetKeyStruct(int idx);
 public:
-  cDeCSA(void);
+  cDeCSA(int CardIndex);
   ~cDeCSA();
   bool Decrypt(unsigned char *data, int len, bool force);
-  bool SetDescr(ca_descr_t *ca_descr);
+  bool SetDescr(ca_descr_t *ca_descr, bool initial);
   bool SetCaPid(ca_pid_t *ca_pid);
+  void SetActive(bool on);
   };
 
-cDeCSA::cDeCSA(void)
+cDeCSA::cDeCSA(int CardIndex)
 {
+  cardindex=CardIndex;
   cs=get_suggested_cluster_size();
-  PRINTF(L_CORE_CSA,"clustersize=%d rangesize=%d",cs,cs*2+5);
+  PRINTF(L_CORE_CSA,"%d: clustersize=%d rangesize=%d",cardindex,cs,cs*2+5);
   range=MALLOC(unsigned char *,(cs*2+5));
   memset(keys,0,sizeof(keys));
+  memset(even_odd,0,sizeof(even_odd));
   memset(pidmap,0,sizeof(pidmap));
 }
 
@@ -2329,55 +2345,67 @@ cDeCSA::~cDeCSA()
   free(range);
 }
 
+void cDeCSA::SetActive(bool on)
+{
+  active=on;
+  PRINTF(L_CORE_CSA,"%d: set active %s",cardindex,active?"on":"off");
+}
+
 bool cDeCSA::GetKeyStruct(int idx)
 {
   if(!keys[idx]) keys[idx]=get_key_struct();
   return keys[idx]!=0;
 }
 
-bool cDeCSA::SetDescr(ca_descr_t *ca_descr)
+bool cDeCSA::SetDescr(ca_descr_t *ca_descr, bool initial)
 {
-  cMutexLock lock(this);
+  cMutexLock lock(&mutex);
   int idx=ca_descr->index;
   if(idx<MAX_CSA_IDX && GetKeyStruct(idx)) {
-    if(ca_descr->parity==0) { // even key
-      set_even_control_word(keys[idx],ca_descr->cw);
-      PRINTF(L_CORE_CSA,"even key set");
-      }
-    else {                    // odd key
-      set_odd_control_word(keys[idx],ca_descr->cw);
-      PRINTF(L_CORE_CSA,"odd key set");
+    if(!initial && active && ca_descr->parity==(even_odd[idx]&0x40)>>6) {
+      PRINTF(L_CORE_CSA,"%d.%d: %s key in use",cardindex,idx,ca_descr->parity?"odd":"even");
+      if(wait.TimedWait(mutex,100)) PRINTF(L_CORE_CSA,"%d.%d: successfully waited for release",cardindex,idx);
+      else PRINTF(L_CORE_CSA,"%d.%d: timed out. setting anyways",cardindex,idx);
       }
+    PRINTF(L_CORE_CSA,"%d.%d: %s key set",cardindex,idx,ca_descr->parity?"odd":"even");
+    if(ca_descr->parity==0) set_even_control_word(keys[idx],ca_descr->cw);
+    else                    set_odd_control_word(keys[idx],ca_descr->cw);
     }
   return true;
 }
 
 bool cDeCSA::SetCaPid(ca_pid_t *ca_pid)
 {
-  cMutexLock lock(this);
+  cMutexLock lock(&mutex);
   if(ca_pid->index<MAX_CSA_IDX && ca_pid->pid<MAX_CSA_PIDS) {
     pidmap[ca_pid->pid]=ca_pid->index;
-    PRINTF(L_CORE_CSA,"set pid %04x to index %d",ca_pid->pid,ca_pid->index);
+    PRINTF(L_CORE_CSA,"%d.%d: set pid %04x",cardindex,ca_pid->index,ca_pid->pid);
     }
   return true;
 }
 
 bool cDeCSA::Decrypt(unsigned char *data, int len, bool force)
 {
-  cMutexLock lock(this);
+  cMutexLock lock(&mutex);
   int r=-2, ccs=0, currIdx=-1;
   bool newRange=true;
   range[0]=0;
   len-=(TS_SIZE-1);
   for(int l=0; l<len; l+=TS_SIZE) {
     if(data[l]!=TS_SYNC_BYTE) {       // let higher level cope with that
-      PRINTF(L_CORE_CSA,"garbage in TS buffer");
+      PRINTF(L_CORE_CSA,"%d: garbage in TS buffer",cardindex);
       break;
       }
-    if(data[l+3]&0xC0) {              // encrypted
+    unsigned int ev_od=data[l+3]&0xC0;
+    if(ev_od==0x80 || ev_od==0xC0) { // encrypted
       int idx=pidmap[((data[l+1]<<8)+data[l+2])&(MAX_CSA_PIDS-1)];
       if(currIdx<0 || idx==currIdx) { // same or no index
         currIdx=idx;
+        if(ccs==0 && ev_od!=even_odd[idx]) {
+          even_odd[idx]=ev_od;
+          wait.Broadcast();
+          PRINTF(L_CORE_CSA,"%d.%d: change to %s key",cardindex,idx,(ev_od&0x40)?"odd":"even");
+          }
         if(newRange) {
           r+=2; newRange=false;
           range[r]=&data[l];
@@ -2386,29 +2414,25 @@ bool cDeCSA::Decrypt(unsigned char *data, int len, bool force)
         range[r+1]=&data[l+TS_SIZE];
         if(++ccs>=cs) break;
         }
-      else {                          // other index, create hole
-        PRINTF(L_CORE_CSA,"other index. current=%d idx=%d",currIdx,idx);
-        newRange=true;
-        }
+      else newRange=true;             // other index, create hole
       }
     else {                            // unencrypted
       // nothing, we don't create holes for unencrypted packets
       }
     }
-  if(force) PRINTF(L_CORE_CSA,"forced");
-  if(r>cs*2) PRINTF(L_CORE_CSA,"range overflow");
-  if(r<0) PRINTF(L_CORE_CSA,"nothing to decrypt");
-  if(r>=0 && (ccs>=cs || force) && GetKeyStruct(currIdx)) { // we have some range
-    LBSTARTF(L_CORE_CSA);
-    LBPUT("decrypting %3d packets (ccs=%3d cs=%3d %s)",ccs,ccs,cs,ccs>=cs?"OK ":"INC");
-    int n=decrypt_packets(keys[currIdx],range);
-    LBPUT(" -> %3d packets decrypted\n",n);
-    if(n>0) return true;
-    LBEND();
-    }
-  else {
-    if(ccs<cs && !force)
-      PRINTF(L_CORE_CSA,"incomplete cluster ccs=%3d cs=%3d",ccs,cs);
+  if(r>=0) {                          // we have some range
+    if(ccs>=cs || force) {
+      if(GetKeyStruct(currIdx)) {
+        int n=decrypt_packets(keys[currIdx],range);
+#ifdef DEBUG_CSA
+        PRINTF(L_CORE_CSA,"%d.%d: decrypting ccs=%3d cs=%3d %s -> %3d decrypted",cardindex,currIdx,ccs,cs,ccs>=cs?"OK ":"INC",n);
+#endif
+        if(n>0) return true;
+        }
+      }
+#ifdef DEBUG_CSA
+    else PRINTF(L_CORE_CSA,"%d.%d: incomplete cluster ccs=%3d cs=%3d",cardindex,currIdx,ccs,cs);
+#endif
     }
   return false;
 }
@@ -2440,14 +2464,16 @@ cDeCsaTSBuffer::cDeCsaTSBuffer(int File, int Size, int CardIndex, cDeCSA *DeCsa,
   f=File; cardIndex=CardIndex; decsa=DeCsa; scActive=ScActive;
   delivered=false;
   lastP=0; lastCount=0;
-  ringBuffer=new cRingBufferLinear(Size,TS_SIZE,true,"TS");
+  ringBuffer=new cRingBufferLinear(Size,TS_SIZE,true,"FFdecsa-TS");
   ringBuffer->SetTimeouts(100,100);
+  if(decsa) decsa->SetActive(true);
   Start();
 }
 
 cDeCsaTSBuffer::~cDeCsaTSBuffer()
 {
   Cancel(3);
+  if(decsa) decsa->SetActive(false);
   delete ringBuffer;
 }
 
@@ -2502,14 +2528,21 @@ uchar *cDeCsaTSBuffer::Get(void)
   return NULL;
 }
 
+#endif //SASC
+
 // -- cScDvbDevice -------------------------------------------------------------
 
 int cScDvbDevice::budget=0;
 
+#ifndef SASC
+
 cScDvbDevice::cScDvbDevice(int n, int cafd)
 :cDvbDevice(n)
 {
-  decsa=0; tsBuffer=0; cam=0; ciadapter=0; hwciadapter=0;
+  decsa=0; tsBuffer=0; cam=0;
+#if APIVERSNUM >= 10500
+  ciadapter=0; hwciadapter=0;
+#endif
   memset(lrucaid,0,sizeof(lrucaid));
   fd_ca=cafd; fd_ca2=dup(fd_ca); fd_dvr=-1;
   softcsa=(fd_ca<0);
@@ -2522,14 +2555,18 @@ cScDvbDevice::~cScDvbDevice()
   EarlyShutdown();
   delete decsa;
   if(fd_ca>=0) close(fd_ca);
+#if APIVERSNUM >= 10500
   if(fd_ca2>=0) close(fd_ca2);
+#endif
 }
 
 void cScDvbDevice::EarlyShutdown(void)
 {
+#if APIVERSNUM >= 10500
   SetCamSlot(0);
   delete ciadapter; ciadapter=0;
   delete hwciadapter; hwciadapter=0;
+#endif
   if(cam) cam->Stop();
   delete cam; cam=0;
 }
@@ -2537,6 +2574,8 @@ void cScDvbDevice::EarlyShutdown(void)
 void cScDvbDevice::LateInit(void)
 {
   int n=CardIndex();
+  if(DeviceNumber()!=n)
+    PRINTF(L_GEN_ERROR,"CardIndex - DeviceNumber mismatch! Put SC plugin first on VDR commandline!");
   if(softcsa) {
     if(HasDecoder()) PRINTF(L_GEN_ERROR,"Card %d is a full-featured card but no ca device found!",n);
     }
@@ -2545,12 +2584,20 @@ void cScDvbDevice::LateInit(void)
     softcsa=true;
     }
   
+#if APIVERSNUM >= 10500
   if(fd_ca2>=0) hwciadapter=cDvbCiAdapter::CreateCiAdapter(this,fd_ca2);
   cam=new cCam(this,n);
   ciadapter=new cScCiAdapter(this,n,cam);
+#else
+  if(fd_ca2>=0) {
+    ciHandler=cCiHandler::CreateCiHandler(fd_ca2);
+    if(!ciHandler) close(fd_ca2);
+    }
+  cam=ScSetup.CapCheck(n) ? new cCam(this,n):0;
+#endif
   if(softcsa) {
     PRINTF(L_GEN_INFO,"Using software decryption on card %d",n);
-    decsa=new cDeCSA;
+    decsa=new cDeCSA(n);
     }
 }
 
@@ -2564,6 +2611,8 @@ void cScDvbDevice::Shutdown(void)
 
 void cScDvbDevice::Startup(void)
 {
+  if(ScSetup.ForceTransfer)
+    SetTransferModeForDolbyDigital(2);
   for(int n=cDevice::NumDevices(); --n>=0;) {
     cScDvbDevice *dev=dynamic_cast<cScDvbDevice *>(cDevice::GetDevice(n));
     if(dev) dev->LateInit();
@@ -2604,6 +2653,10 @@ bool cScDvbDevice::Initialize(void)
     PRINTF(L_GEN_ERROR,"Failed to locate VDR symbols. Plugin not operable");
     return false;
     }
+  if(NumDevices()>0) {
+    PRINTF(L_GEN_ERROR,"Number of devices != 0 on init. Put SC plugin first on VDR commandline! Aborting.");
+    return false;
+    }
   *vdr_nci=0; *vdr_ud=vdr_save_ud;
 
   int i, found=0;
@@ -2645,11 +2698,13 @@ bool cScDvbDevice::HasCi(void)
 }
 #endif
 
+#if APIVERSNUM >= 10500
 bool cScDvbDevice::Ready(void)
 {
   return (ciadapter   ? ciadapter->Ready():true) &&
          (hwciadapter ? hwciadapter->Ready():true);
 }
+#endif
 
 bool cScDvbDevice::SetPid(cPidHandle *Handle, int Type, bool On)
 {
@@ -2660,25 +2715,30 @@ bool cScDvbDevice::SetPid(cPidHandle *Handle, int Type, bool On)
 bool cScDvbDevice::SetChannelDevice(const cChannel *Channel, bool LiveView)
 {
   lruMutex.Lock();
-  int i=FindLRUPrg(Channel->Sid());
-  if(i<0) {
-    memmove(&lrucaid[1],&lrucaid[0],sizeof(struct LruCaid)*(MAX_LRU_CAID-1));
-    i=0;
-    }
-  const int *c=Channel->Caids();
-  int j;
-  for(j=0; j<MAXCAIDS && *c; j++) lrucaid[i].caids[j]=*c++;
-  lrucaid[i].caids[j]=0;
-  lrucaid[i].prg=Channel->Sid();
+  int i=FindLRUPrg(Channel->Source(),Channel->Transponder(),Channel->Sid());
+  if(i<0) i=MAX_LRU_CAID-1;
+  if(i>0) memmove(&lrucaid[1],&lrucaid[0],sizeof(struct LruCaid)*i);
+#if APIVERSNUM >= 10500
+  const caid_t *c=Channel->Caids();
+  for(i=0; i<MAXCAIDS && *c; i++) lrucaid[0].caids[i]=*c++;
+  lrucaid[0].caids[i]=0;
+#else
+  for(i=0; i<=MAXCAIDS; i++) if((lrucaid[0].caids[i]=Channel->Ca(i))==0) break;
+#endif
+  lrucaid[0].src=Channel->Source();
+  lrucaid[0].tr=Channel->Transponder();
+  lrucaid[0].prg=Channel->Sid();
   lruMutex.Unlock();
   if(cam) cam->Tune(Channel);
-  return cDvbDevice::SetChannelDevice(Channel,LiveView);
+  bool ret=cDvbDevice::SetChannelDevice(Channel,LiveView);
+  if(ret && cam) cam->PostTune();
+  return ret;
 }
 
-bool cScDvbDevice::GetPrgCaids(int prg, int *c)
+bool cScDvbDevice::GetPrgCaids(int source, int transponder, int prg, caid_t *c)
 {
   cMutexLock lock(&lruMutex);
-  int i=FindLRUPrg(prg);
+  int i=FindLRUPrg(source,transponder,prg);
   if(i>=0) {
     for(int j=0; j<MAXCAIDS && lrucaid[i].caids[j]; j++) *c++=lrucaid[i].caids[j];
     *c=0;
@@ -2687,18 +2747,68 @@ bool cScDvbDevice::GetPrgCaids(int prg, int *c)
   return false;
 }
 
-int cScDvbDevice::FindLRUPrg(int prg)
+int cScDvbDevice::FindLRUPrg(int source, int transponder, int prg)
 {
-  for(int i=0; i<MAX_LRU_CAID; i++) if(lrucaid[i].prg==prg) return i;
+  for(int i=0; i<MAX_LRU_CAID; i++)
+    if(lrucaid[i].src==source && lrucaid[i].tr==transponder && lrucaid[i].prg==prg) return i;
   return -1;
 }
 
+#if APIVERSNUM < 10500
+int cScDvbDevice::ProvidesCa(const cChannel *Channel) const
+{
+  if(cam && Channel->Ca()>=CA_ENCRYPTED_MIN) {
+    int j;
+    caid_t ids[MAXCAIDS+1];
+    for(j=0; j<=MAXCAIDS; j++) if((ids[j]=Channel->Ca(j))==0) break;
+    if(cSystems::Provides(ids,!softcsa)>0) return 2;
+    }
+  return cDvbDevice::ProvidesCa(Channel);
+}
+
+bool cScDvbDevice::CiAllowConcurrent(void) const
+{
+  return softcsa || ScSetup.ConcurrentFF>0;
+}
+
+void cScDvbDevice::CiStartDecrypting(void)
+{
+  if(cam) {
+    cSimpleList<cPrg> prgList;
+    for(cCiCaProgramData *p=ciProgramList.First(); p; p=ciProgramList.Next(p)) {
+      if(p->modified) {
+        cPrg *prg=new cPrg(p->programNumber,cam->HasPrg(p->programNumber));
+        if(prg) {
+          for(cCiCaPidData *q=p->pidList.First(); q; q=p->pidList.Next(q)) {
+            if(q->active)
+              prg->pids.Add(new cPrgPid(q->streamType,q->pid));
+            }
+          prgList.Add(prg);
+          }
+        p->modified=false;
+        }
+      }
+    for(int loop=1; loop<=2; loop++) // first delete, then add
+      for(cPrg *prg=prgList.First(); prg; prg=prgList.Next(prg))
+        if((loop==1)!=(prg->pids.Count()>0))
+          cam->AddPrg(prg);
+    }
+  cDvbDevice::CiStartDecrypting();
+}
+#endif //APIVERSNUM < 10500
+
 bool cScDvbDevice::OpenDvr(void)
 {
   CloseDvr();
   fd_dvr=DvbOpen(DEV_DVB_DVR,CardIndex(),O_RDONLY|O_NONBLOCK,true);
-  if(fd_dvr>=0)
-    tsBuffer=new cDeCsaTSBuffer(fd_dvr,MEGABYTE(2),CardIndex()+1,decsa,dynamic_cast<cScCamSlot *>(CamSlot())!=0);
+  if(fd_dvr>=0) {
+#if APIVERSNUM >= 10500
+    bool active=dynamic_cast<cScCamSlot *>(CamSlot())!=0;
+#else
+    bool active=cam && softcsa;
+#endif
+    tsBuffer=new cDeCsaTSBuffer(fd_dvr,MEGABYTE(2),CardIndex()+1,decsa,active);
+    }
   return fd_dvr>=0;
 }
 
@@ -2714,13 +2824,13 @@ bool cScDvbDevice::GetTSPacket(uchar *&Data)
   return false;
 }
 
-bool cScDvbDevice::SetCaDescr(ca_descr_t *ca_descr)
+bool cScDvbDevice::SetCaDescr(ca_descr_t *ca_descr, bool initial)
 {
   if(!softcsa) {
     cMutexLock lock(&cafdMutex);
     return ioctl(fd_ca,CA_SET_DESCR,ca_descr)>=0;
     }
-  else if(decsa) return decsa->SetDescr(ca_descr);
+  else if(decsa) return decsa->SetDescr(ca_descr,initial);
   return false;
 }
 
@@ -2810,3 +2920,59 @@ void cScDvbDevice::DumpAV7110(void)
       }
     }
 }
+
+#else //SASC
+
+cScDvbDevice::cScDvbDevice(int n, int cafd)
+:cDvbDevice(n)
+{
+  softcsa=false;
+  cam=new cCam(this,n);
+}
+
+cScDvbDevice::~cScDvbDevice()
+{
+  delete cam;
+}
+
+void cScDvbDevice::Shutdown(void)
+{}
+
+void cScDvbDevice::Startup(void)
+{}
+
+void cScDvbDevice::SetForceBudget(int n)
+{}
+
+bool cScDvbDevice::ForceBudget(int n)
+{
+   return true;
+}
+
+void cScDvbDevice::Capture(void)
+{}
+
+bool cScDvbDevice::Initialize(void)
+{
+  return true;
+}
+
+bool cScDvbDevice::GetPrgCaids(int source, int transponder, int prg, caid_t *c)
+{
+  return false;
+}
+
+bool cScDvbDevice::SetCaDescr(ca_descr_t *ca_descr, bool initial)
+{
+  return false;
+}
+
+bool cScDvbDevice::SetCaPid(ca_pid_t *ca_pid)
+{
+  return false;
+}
+
+void cScDvbDevice::DumpAV7110(void)
+{}
+
+#endif //SASC
