@@ -72,13 +72,22 @@ cTtxtSubsDisplay::cTtxtSubsDisplay(void)
   mPageState(invalid),
   mMag(0),
   mNo(0),
+#if defined(APIVERSNUM) && APIVERSNUM < 10509
   mDoDisplay(0),
+#else
+  mDoDisplay(1),
+#endif
   mOsd(NULL),
   mOsdLock(),
   mLastDataTime(NULL)
 {
   memset(&page.data, 0, sizeof(page.data));
   mLastDataTime = (struct timeval *) calloc(1, sizeof(*mLastDataTime));
+#if defined(APIVERSNUM) && APIVERSNUM >= 10503
+  mOsdFont = cFont::CreateFont(Setup.FontOsd, globals.mFontSize);
+  if (!mOsdFont || !mOsdFont->Height())
+#endif
+     mOsdFont = cFont::GetFont(fontOsd);
 }
 
 
@@ -91,6 +100,10 @@ cTtxtSubsDisplay::~cTtxtSubsDisplay(void)
     mOsd = NULL;
     delete tmp;
   }
+#if defined(APIVERSNUM) && APIVERSNUM >= 10503
+  if(mOsdFont && (mOsdFont != cFont::GetFont(fontOsd)))
+    delete mOsdFont;
+#endif
 }
 
 
@@ -382,6 +395,11 @@ void cTtxtSubsDisplay::ShowOSD(void)
   cOSDSelfMemoryLock selfmem(&gSelfMem);
   cMutexLock lock(&mOsdLock);
 
+  if(!globals.mRealDoDisplay) {
+    //dprint("NOT displaying subtitles because disabled!\n");
+    return;
+  }
+ 
   if(!mDoDisplay) {
     //dprint("NOT displaying subtitles because of other OSD activities!\n");
     return;
@@ -404,17 +422,23 @@ void cTtxtSubsDisplay::ShowOSD(void)
     delete tmp;
   }
 
+#if defined(APIVERSNUM) && APIVERSNUM < 10509
   if (cOsd::IsOpen()) {
      //dprint("NOT displaying subtitles because of other OSD activities!\n");
      return;
      }
   else {
      mOsd = cOsdProvider::NewOsd(SCREENLEFT, SCREENTOP);
+#else
+     mOsd = cOsdProvider::NewOsd(SCREENLEFT, SCREENTOP, 20); // level 20
+#endif
      if(!mOsd) {
        //dprint("Error: cOsdProvider::NewOsd() returned NULL!\n");
        return;
        }
+#if defined(APIVERSNUM) && APIVERSNUM < 10509
      }
+#endif
 
 #if defined(APIVERSNUM) && APIVERSNUM < 10503
   cFont::SetCode(I18nCharSets()[globals.i18nLanguage()]);
@@ -425,7 +449,7 @@ void cTtxtSubsDisplay::ShowOSD(void)
   for(i = 0; i < rowcount; i++) {
     int w = 0;
     int left = SIDEMARGIN;
-    w = cFont::GetFont(fontOsd)->Width(buf[i]) + 2 * TEXTX;
+    w = mOsdFont->Width(buf[i]) + 2 * TEXTX;
     if(w % 4)
       w += 4 - (w % 4);
     switch(globals.textPos()) {
@@ -452,7 +476,7 @@ void cTtxtSubsDisplay::ShowOSD(void)
     for(i = 0; i < rowcount; i++) {
       int w = 0;
       int left = SIDEMARGIN;
-      w = cFont::GetFont(fontOsd)->Width(buf[i]) + 2 * TEXTX;
+      w = mOsdFont->Width(buf[i]) + 2 * TEXTX;
       if(w % 4)
         w += 4 - (w % 4);
       switch(globals.textPos()) {
@@ -464,7 +488,7 @@ void cTtxtSubsDisplay::ShowOSD(void)
           break;
         }
       mOsd->DrawRectangle(left, y, left + w, y + ROWH, getcolor(globals.bgColor()));
-      mOsd->DrawText(left + TEXTX, y + TEXTY, buf[i], getcolor(globals.fgColor()), getcolor(globals.bgColor()), cFont::GetFont(fontOsd));
+      mOsd->DrawText(left + TEXTX, y + TEXTY, buf[i], getcolor(globals.fgColor()), getcolor(globals.bgColor()), mOsdFont);
       //dprint("%d/%d (%d,%d) (%d,%d): %s\n", i, rowcount-1, areas[i].x1, areas[i].y1, left + TEXTX, y + TEXTY, buf[i]);
       y += (ROWINCR + globals.lineSpacing());
       }
