@@ -9,8 +9,7 @@
 #include "ttxtsubsi18n.h"
 #include <stdio.h>
 
-const tI18nPhrase Phrases[] =
-{
+const tI18nPhrase Phrases[] = {
   { "Teletext subtitles", // English
     "Videotextuntertitel", // German - Tobias Grimm 2004-05-21
     "", // Slovenian
@@ -473,6 +472,27 @@ const tI18nPhrase Phrases[] =
     "", // Eesti
     "", // Dansk
   },
+  { "Font Size (pixel)", // English
+    "", // German
+    "", // Slovenian
+    "", // Italian
+    "", // Dutch
+    "", // Portuguese
+    "", // French
+    "", // Norwegian
+    "Kirjasintyypin koko (px)", // Finnish
+    "", // Polish
+    "", // Spanish
+    "", // Greek
+    "", // Swedish
+    "", // Romanian
+    "", // Hungarian
+    "", // Catalenian
+    "", // Russian
+    "", // Croatian
+    "", // Eesti
+    "", // Dansk
+  },
   { "Text Color", // English
     "Textfarbe", // German
     "", // Slovenian
@@ -725,27 +745,6 @@ const tI18nPhrase Phrases[] =
     "", // Eesti
     "", // Dansk
   },
-  { "Background", // English
-    "Hintergrund", // German
-    "", // Slovenian
-    "Sfondo", // Italian
-    "", // Dutch
-    "", // Portuguese
-    "Fond", // French
-    "", // Norwegian
-    "tausta", // Finnish
-    "", // Polish
-    "Fondo", // Spanish
-    "", // Greek
-    "bakgrund", // Swedish
-    "", // Romanian
-    "", // Hungarian
-    "Fons", // Catalenian
-    " ", // Russian
-    "", // Croatian
-    "", // Eesti
-    "", // Dansk
-  },
   { "Custom Color", // English
     "Benutzerdefinierte Farbe", // German
     "", // Slovenian
