@@ -17,6 +17,9 @@ static const tColor tint		= (tColor) 0xFF002850;
 static const tColor highlight		= (tColor) 0xFF505050;
 static const tColor lightgrey		= (tColor) 0xFFAAAAAA;
 
+#if VDRVERSNUM >= 10503
+#define fontYaepg			fontSml
+#endif
 #define fontChanBox			fontYaepg
 #define fontTimeBox			fontYaepg
 #define fontEventBox			fontYaepg
@@ -42,15 +45,6 @@ static const tColor lightgrey		= (tColor) 0xFFAAAAAA;
 #define ARROW_LEFT			1
 #define ARROW_RIGHT			2
 
-/* Order of channels (UP or DOWN) */
-#define CHANNEL_ORDER_UP      0
-#define CHANNEL_ORDER_DOWN      1
-
-/* Manner in which channel is changed while in YAEPG */
-#define CHANNEL_CHANGE_MANUAL      0
-#define CHANNEL_CHANGE_MANUAL_INEPG   1
-#define CHANNEL_CHANGE_AUTO_INEPG   2
-
 /*
  * PAL/NTSC Output
  */
@@ -71,13 +65,10 @@ static const tColor lightgrey		= (tColor) 0xFFAAAAAA;
 int iTvFormat				= TV_NTSC;
 int iChannelCount			= CHANNELS_NTSC;
 int iHideMenuEntry			= false;
-int iChannelChange                      = CHANNEL_CHANGE_MANUAL;
+int bAutoChannelChange			= false;
 int iTimeFormat				= TIME_24HR;
-int iChannelOrder                       = CHANNEL_ORDER_DOWN;
 static const char *TV_FORMATS[2]	= { "NTSC", "PAL" };
 static const char *TIME_FORMATS[2]	= { "24", "12" };
-static const char *CH_ORDER_FORMATS[2]   = { "UP", "DOWN" };
-static const char *CH_CHANGE_MODES[3]   = { "MANUAL", "MANUAL IN EPG", "AUTO IN EPG" };
 
 int iOutputResW				= TV_NTSC_W;  
 int iOutputResH				= TV_NTSC_H;  
@@ -1181,7 +1172,7 @@ cMessageBox::Draw(void)
 
 class cYaepg: public cOsdObject {
 private:
-	cOsd			*osd;
+	cOsd		*osd;
 	tArea			areas[3];
 	cBitmap			*EPGBitmap;
 	cBitmap			*InfoBitmap;
@@ -1197,9 +1188,7 @@ private:
 	cTimeBox		*timebox[4];
 	cChanBox		*chans[6]; //TODO
 	sEventRow		eventgrid[6]; //TODO
-        
-	int                     last_channel, first_channel;
-	
+
 	/*
 	 * Info window stuff.
 	 */
@@ -1261,7 +1250,6 @@ public:
 	void MoveCursor(int);
 	void ChangeChan(int);
 	int ChangeTime(int);
-	void GetChannelListEnds(void);
 	void UpdateTime(void);
 	static void LogMsg(eYlog_t, const char *fmt, ...);
 };
@@ -1406,16 +1394,6 @@ cYaepg::SwitchToCurrentChannel(void)
 	return;	
 }
 
-void
-cYaepg::GetChannelListEnds()
-{
-        // Set the first and last channel numbers
-        cChannel *c = Channels.First();
-        first_channel = c->Number();
-        c = Channels.Last();
-        last_channel = c->Number();
-}
-
 void
 cYaepg::Draw(void)
 {
@@ -1465,21 +1443,14 @@ cYaepg::Draw(void)
 	 */
 	cx = 36, cy = 22;
 	c = Channels.GetByNumber(startChan);
-	GetChannelListEnds();
 	for (i = 0; i < iChannelCount; i++) {
 		if (c != NULL) {
 			chans[i] = new cChanBox(c);
 			chans[i]->Draw(cx, cy, EPGBitmap);
 			cy += chans[i]->Height() + 2;
 		}
-                if (iChannelOrder == CHANNEL_ORDER_UP) {
-                   while ((c = (cChannel *)c->Prev()) && (c->GroupSep()));
-                   if (c == NULL)
-                      c = Channels.GetByNumber(last_channel);
-                } else {
-                   while ((c = (cChannel *)c->Next()) && (c->GroupSep()));
-                   if (c == NULL) 
-                      c = Channels.GetByNumber(first_channel);
+		while ((c = (cChannel *)c->Next()) && (c->GroupSep())) {
+			;
 		}
 	}
 
@@ -1671,11 +1642,7 @@ cYaepg::ProcessKey(eKeys Key)
 
 		case kBlue:
 			/* Can this be fixed to work with NTSC ? */
-#if VDRVERSNUM >= 10338
-			cDevice::PrimaryDevice()->GrabImageFile("img.jpg",
-#else
-			cDevice::PrimaryDevice()->GrabImage("img.jpg",
-#endif
+			cDevice::PrimaryDevice()->GrabImageFile("/tmp/img.jpg",
 					true, 256, -1, -1);
 			state = osContinue;
 			break;
@@ -1696,7 +1663,7 @@ cYaepg::ProcessKey(eKeys Key)
 			MoveCursor(CURSOR_UP);
 			Draw();
 
-			if (iChannelChange == CHANNEL_CHANGE_AUTO_INEPG) {
+			if (bAutoChannelChange) {
 				SwitchToCurrentChannel();
                         }
 
@@ -1707,7 +1674,7 @@ cYaepg::ProcessKey(eKeys Key)
 			MoveCursor(CURSOR_DOWN);
 			Draw(); 
 
-			if (iChannelChange == CHANNEL_CHANGE_AUTO_INEPG) {
+			if (bAutoChannelChange) {
 				SwitchToCurrentChannel();
 			}
 			
@@ -1721,10 +1688,7 @@ cYaepg::ProcessKey(eKeys Key)
 			 */
 			if (selEvent->StartTime() <= time(NULL)) {
 				SwitchToCurrentChannel();
-				if (iChannelChange == CHANNEL_CHANGE_MANUAL_INEPG)
-				   state = osContinue;
-				else
-				   state = osEnd;
+				state = osEnd;
 			}
 			else {
 				recDlgBox = new cRecDlgBox(selEvent);
@@ -1734,7 +1698,7 @@ cYaepg::ProcessKey(eKeys Key)
 
 		case kRed:
 		case kChanUp:
-			ChangeChan((iChannelOrder == CHANNEL_ORDER_UP ? 1 : -1) * iChannelCount * 1);
+			ChangeChan(iChannelCount * -1);
 			Draw();
 			state = osContinue;
 			break;
@@ -1746,7 +1710,7 @@ cYaepg::ProcessKey(eKeys Key)
 
 		case kYellow:
 		case kChanDn:
-			ChangeChan((iChannelOrder == CHANNEL_ORDER_UP ? 1 : -1) * iChannelCount * -1);
+			ChangeChan(iChannelCount);
 			Draw();
 			state = osContinue;
 			break;
@@ -1841,14 +1805,13 @@ cYaepg::ChangeChan(int change)
 	cChannel *c;
 
 	c = Channels.GetByNumber(startChan);
-	GetChannelListEnds();
 	if (change > 0) {
 		for (int i = 0; i < change; i++) {
 			while ((c = (cChannel *)c->Next()) && (c->GroupSep())) {
 				;
 			}
 			if (c == NULL) {
-				c = Channels.GetByNumber(first_channel);
+				return;
 			}
 			startChan = c->Number();
 			/*
@@ -1861,16 +1824,12 @@ cYaepg::ChangeChan(int change)
 		}
 	}
 	else if (change < 0) {
-	        if (startChan < first_channel) {
-			startChan = last_channel;
-			return;
-		}
 		for (int i = 0; i > change; i--) {
 			while ((c = (cChannel *)c->Prev()) && (c->GroupSep())) {
 				;
 			}
 			if (c == NULL) {
-				c = Channels.GetByNumber(last_channel);
+				return;
 			}
 			startChan = c->Number();
 		}
@@ -1902,7 +1861,7 @@ cYaepg::MoveCursor(int dir)
 	switch (dir) {
 	case CURSOR_UP:
 		if (cursorY == 0) {
-			ChangeChan(1 * (iChannelOrder == CHANNEL_ORDER_UP ? 1 : -1));
+			ChangeChan(-1);
 		}
 		else {
 			cursorY--;
@@ -1911,7 +1870,7 @@ cYaepg::MoveCursor(int dir)
 
 	case CURSOR_DOWN:
 		if (cursorY == iChannelCount - 1) {
-			ChangeChan(-1 * (iChannelOrder == CHANNEL_ORDER_UP ? 1 : -1));
+			ChangeChan(+1);
 		}
 		else {
 			cursorY++;
@@ -2039,9 +1998,8 @@ class cMenuSetupYaepg : public cMenuSetupPage {
 private:
 	int iNewTvFormat;
 	int iNewHideMenuEntry;
-	int iNewChannelChange;
+	int bNewAutoChannelChange;
 	int iNewTimeFormat;
-	int iNewChannelOrder;
 
 protected:
 	virtual void Store(void);
@@ -2118,30 +2076,26 @@ void cMenuSetupYaepg::Store(void)
 {
 	iTvFormat		= iNewTvFormat;
 	iHideMenuEntry		= iNewHideMenuEntry;
-	iChannelChange          = iNewChannelChange;
+	bAutoChannelChange	= bNewAutoChannelChange;
 	iTimeFormat		= iNewTimeFormat;
-	iChannelOrder           = iNewChannelOrder;
 	
 	SetupStore("Format",		iTvFormat);
 	SetupStore("HideMenuEntry",	iHideMenuEntry);
-	SetupStore("ChannelChange",     iChannelChange);
+	SetupStore("AutoChannelChange",	bAutoChannelChange);
 	SetupStore("TimeFormat",	iTimeFormat);
-	SetupStore("ChannelOrder",      iChannelOrder);
 }
 
 cMenuSetupYaepg::cMenuSetupYaepg(void)
 {
 	iNewTvFormat		= iTvFormat;
 	iNewHideMenuEntry	= iHideMenuEntry;
-	iNewChannelChange       = iChannelChange;
+	bNewAutoChannelChange	= bAutoChannelChange;
 	iNewTimeFormat		= iTimeFormat;
-	iNewChannelOrder        = iChannelOrder;
 
 	Add(new cMenuEditStraItem (tr("TV output format"), &iNewTvFormat, 2, TV_FORMATS));
 	Add(new cMenuEditBoolItem (tr("Hide mainmenu entry"), &iNewHideMenuEntry));
-	Add(new cMenuEditStraItem (tr("Channel Change"), &iNewChannelChange, 3, CH_CHANGE_MODES));
+	Add(new cMenuEditBoolItem (tr("Change channel automatically"), &bNewAutoChannelChange));
 	Add(new cMenuEditStraItem (tr("Time format"), &iNewTimeFormat, 2, TIME_FORMATS));
-	Add(new cMenuEditStraItem (tr("Channel Order"), &iNewChannelOrder, 2, CH_ORDER_FORMATS));
 }
 
 cMenuSetupPage *cPluginYaepg::SetupMenu(void)
@@ -2159,15 +2113,12 @@ bool cPluginYaepg::SetupParse(const char *Name, const char *Value)
 	else if (!strcasecmp(Name, "HideMenuEntry")) {
 		iHideMenuEntry = atoi(Value);
 	}
-	else if (!strcasecmp(Name, "ChannelChange")) {
-		iChannelChange = atoi(Value);
+	else if (!strcasecmp(Name, "AutoChannelChange")) {
+		bAutoChannelChange = atoi(Value);
 	}
 	else if (!strcasecmp(Name, "TimeFormat")) {
 		iTimeFormat = atoi(Value);
 	}
-	else if (!strcasecmp(Name, "ChannelOrder")) {
-		iChannelOrder = atoi(Value);
-	}
 	else {
 		return false;
 	}
