@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 1.54 2007/07/21 14:55:01 kls Exp $
+ * $Id: channels.c 1.55 2007/10/12 14:40:53 kls Exp $
  */
 
 #include "channels.h"
@@ -468,7 +468,7 @@ void cChannel::SetPids(int Vpid, int Ppid, int *Apids, char ALangs[][MAXLANGCODE
         q += IntArrayToString(q, Dpids, 10, DLangs);
         }
      *q = 0;
-     const int SBufferSize = MAXSPIDS * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod', +10: paranoia
+     const int SBufferSize = MAXSPIDS * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod', +10: paranoia
      char OldSpidsBuf[SBufferSize];
      char NewSpidsBuf[SBufferSize];
      q = OldSpidsBuf;
