@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c 1.154 2007/06/23 09:42:49 kls Exp $
+ * $Id: config.c 1.157 2007/10/06 14:28:58 kls Exp $
  */
 
 #include "config.h"
@@ -214,7 +214,8 @@ cSetup Setup;
 
 cSetup::cSetup(void)
 {
-  OSDLanguage = 0;
+  CutTimePatchEnabled = 0;
+  strcpy(OSDLanguage, ""); // default is taken from environment
   strcpy(OSDSkin, "sttng");
   strcpy(OSDTheme, "default");
   PrimaryDVB = 1;
@@ -236,6 +237,11 @@ cSetup::cSetup(void)
   MarginStart = 2;
   MarginStop = 10;
   AudioLanguages[0] = -1;
+  DisplaySubtitles = 0;
+  SubtitleLanguages[0] = -1;
+  SubtitleOffset = 0;
+  SubtitleFgTransparency = 0;
+  SubtitleBgTransparency = 0;
   EPGLanguages[0] = -1;
   DoubleEpgTimeDelta = 15;
   DoubleEpgAction = 0;
@@ -259,8 +265,8 @@ cSetup::cSetup(void)
   VideoDisplayFormat = 1;
   VideoFormat = 0;
   UpdateChannels = 5;
+  RecordDolbyDigital = 1;
   UseDolbyDigital = 1;
-  DolbyInRecordings = 1;
   ChannelInfoPos = 0;
   ChannelInfoTime = 5;
   OSDLeft = 54;
@@ -294,6 +300,8 @@ cSetup::cSetup(void)
   ShowRecLength = 0;
   ShowProgressBar = 0;
   MenuCmdPosition = 0;
+  VerboseLNBlog = 0;
+  for (int i = 0; i < MAXDEVICES; i++) CardUsesLNBnr[i] = i + 1;
 }
 
 cSetup& cSetup::operator= (const cSetup &s)
@@ -359,9 +367,9 @@ bool cSetup::Load(const char *FileName)
 
 void cSetup::StoreLanguages(const char *Name, int *Values)
 {
-  char buffer[I18nNumLanguages * 4];
+  char buffer[I18nLanguages()->Size() * 4];
   char *q = buffer;
-  for (int i = 0; i < I18nNumLanguages; i++) {
+  for (int i = 0; i < I18nLanguages()->Size(); i++) {
       if (Values[i] < 0)
          break;
       const char *s = I18nLanguageCode(Values[i]);
@@ -379,7 +387,7 @@ void cSetup::StoreLanguages(const char *Name, int *Values)
 bool cSetup::ParseLanguages(const char *Value, int *Values)
 {
   int n = 0;
-  while (Value && *Value && n < I18nNumLanguages) {
+  while (Value && *Value && n < I18nLanguages()->Size()) {
         char buffer[4];
         strn0cpy(buffer, Value, sizeof(buffer));
         int i = I18nLanguageIndex(buffer);
@@ -394,7 +402,7 @@ bool cSetup::ParseLanguages(const char *Value, int *Values)
 
 bool cSetup::Parse(const char *Name, const char *Value)
 {
-  if      (!strcasecmp(Name, "OSDLanguage"))         OSDLanguage        = atoi(Value);
+  if      (!strcasecmp(Name, "OSDLanguage"))       { strn0cpy(OSDLanguage, Value, sizeof(OSDLanguage)); I18nSetLocale(OSDLanguage); }
   else if (!strcasecmp(Name, "OSDSkin"))             Utf8Strn0Cpy(OSDSkin, Value, MaxSkinName);
   else if (!strcasecmp(Name, "OSDTheme"))            Utf8Strn0Cpy(OSDTheme, Value, MaxThemeName);
   else if (!strcasecmp(Name, "PrimaryDVB"))          PrimaryDVB         = atoi(Value);
@@ -416,6 +424,11 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "MarginStart"))         MarginStart        = atoi(Value);
   else if (!strcasecmp(Name, "MarginStop"))          MarginStop         = atoi(Value);
   else if (!strcasecmp(Name, "AudioLanguages"))      return ParseLanguages(Value, AudioLanguages);
+  else if (!strcasecmp(Name, "DisplaySubtitles"))    DisplaySubtitles   = atoi(Value);
+  else if (!strcasecmp(Name, "SubtitleLanguages"))   return ParseLanguages(Value, SubtitleLanguages);
+  else if (!strcasecmp(Name, "SubtitleOffset"))      SubtitleOffset     = atoi(Value);
+  else if (!strcasecmp(Name, "SubtitleFgTransparency")) SubtitleFgTransparency = atoi(Value);
+  else if (!strcasecmp(Name, "SubtitleBgTransparency")) SubtitleBgTransparency = atoi(Value);
   else if (!strcasecmp(Name, "EPGLanguages"))        return ParseLanguages(Value, EPGLanguages);
   else if (!strcasecmp(Name, "DoubleEpgTimeDelta"))  DoubleEpgTimeDelta = atoi(Value);
   else if (!strcasecmp(Name, "DoubleEpgAction"))     DoubleEpgAction    = atoi(Value);
@@ -439,8 +452,8 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "VideoDisplayFormat"))  VideoDisplayFormat = atoi(Value);
   else if (!strcasecmp(Name, "VideoFormat"))         VideoFormat        = atoi(Value);
   else if (!strcasecmp(Name, "UpdateChannels"))      UpdateChannels     = atoi(Value);
+  else if (!strcasecmp(Name, "RecordDolbyDigital"))  RecordDolbyDigital = atoi(Value);
   else if (!strcasecmp(Name, "UseDolbyDigital"))     UseDolbyDigital    = atoi(Value);
-  else if (!strcasecmp(Name, "DolbyInRecordings"))   DolbyInRecordings  = atoi(Value);
   else if (!strcasecmp(Name, "ChannelInfoPos"))      ChannelInfoPos     = atoi(Value);
   else if (!strcasecmp(Name, "ChannelInfoTime"))     ChannelInfoTime    = atoi(Value);
   else if (!strcasecmp(Name, "OSDLeft"))             OSDLeft            = atoi(Value);
@@ -474,8 +487,21 @@ bool cSetup::Parse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "ShowRecLength"))       ShowRecLength      = atoi(Value);
   else if (!strcasecmp(Name, "ShowProgressBar"))     ShowProgressBar    = atoi(Value);
   else if (!strcasecmp(Name, "MenuCmdPosition"))     MenuCmdPosition    = atoi(Value);
+  else if (!strcasecmp(Name, "CutTimePatchEnabled")) CutTimePatchEnabled= atoi(Value);
   else
-     return false;
+  if (!strcasecmp(Name, "VerboseLNBlog")) VerboseLNBlog = atoi(Value);
+  else {
+     char tmp[20];
+     bool result = false;
+     for (int i = 1; i <= MAXDEVICES; i++) {
+         sprintf(tmp, "Card%dusesLNBnr", i);
+         if (!strcasecmp(Name, tmp)) {
+            CardUsesLNBnr[i - 1] = atoi(Value);
+            result = true;
+            }
+         }
+     return result;
+  }
   return true;
 }
 
@@ -503,6 +529,11 @@ bool cSetup::Save(void)
   Store("MarginStart",        MarginStart);
   Store("MarginStop",         MarginStop);
   StoreLanguages("AudioLanguages", AudioLanguages);
+  Store("DisplaySubtitles",   DisplaySubtitles);
+  StoreLanguages("SubtitleLanguages", SubtitleLanguages);
+  Store("SubtitleOffset",     SubtitleOffset);
+  Store("SubtitleFgTransparency", SubtitleFgTransparency);
+  Store("SubtitleBgTransparency", SubtitleBgTransparency);
   StoreLanguages("EPGLanguages", EPGLanguages);
   Store("DoubleEpgTimeDelta", DoubleEpgTimeDelta);
   Store("DoubleEpgAction",    DoubleEpgAction);
@@ -526,8 +557,8 @@ bool cSetup::Save(void)
   Store("VideoDisplayFormat", VideoDisplayFormat);
   Store("VideoFormat",        VideoFormat);
   Store("UpdateChannels",     UpdateChannels);
+  Store("RecordDolbyDigital", RecordDolbyDigital);
   Store("UseDolbyDigital",    UseDolbyDigital);
-  Store("DolbyInRecordings",  DolbyInRecordings);
   Store("ChannelInfoPos",     ChannelInfoPos);
   Store("ChannelInfoTime",    ChannelInfoTime);
   Store("OSDLeft",            OSDLeft);
@@ -561,6 +592,15 @@ bool cSetup::Save(void)
   Store("ShowRecLength",      ShowRecLength);
   Store("ShowProgressBar",    ShowProgressBar);
   Store("MenuCmdPosition",    MenuCmdPosition);
+  Store("CutTimePatchEnabled",CutTimePatchEnabled);
+  Store("VerboseLNBlog",      VerboseLNBlog);
+  char tmp[20];
+  if (cDevice::NumDevices() > 1) {
+     for (int i = 1; i <= cDevice::NumDevices(); i++) {
+         sprintf(tmp, "Card%dusesLNBnr", i);
+         Store(tmp, CardUsesLNBnr[i - 1]);
+         }
+     }
 
   Sort();
 
