@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c 1.141 2007/06/16 09:31:32 kls Exp $
+ * $Id: device.c 1.145 2007/10/14 13:09:19 kls Exp $
  */
 
 #include "device.h"
@@ -18,6 +18,76 @@
 #include "receiver.h"
 #include "status.h"
 #include "transfer.h"
+#include "diseqc.h"
+
+// --- cLiveSubtitle ---------------------------------------------------------
+
+#define LIVESUBTITLEBUFSIZE  KILOBYTE(100)
+
+class cLiveSubtitle : public cReceiver, public cThread {
+private:
+  cRingBufferLinear *ringBuffer;
+  cRemux *remux;
+protected:
+  virtual void Activate(bool On);
+  virtual void Receive(uchar *Data, int Length);
+  virtual void Action(void);
+public:
+  cLiveSubtitle(int SPid);
+  virtual ~cLiveSubtitle();
+  };
+
+cLiveSubtitle::cLiveSubtitle(int SPid)
+:cReceiver(tChannelID(), -1, SPid)
+,cThread("live subtitle")
+{
+  ringBuffer = new cRingBufferLinear(LIVESUBTITLEBUFSIZE, TS_SIZE * 2, true, "Live Subtitle");
+  int NoPids = 0;
+  int SPids[] = { SPid, 0 };
+  remux = new cRemux(0, &NoPids, &NoPids, SPids);
+}
+
+cLiveSubtitle::~cLiveSubtitle()
+{
+  cReceiver::Detach();
+  delete remux;
+  delete ringBuffer;
+}
+
+void cLiveSubtitle::Activate(bool On)
+{
+  if (On)
+     Start();
+  else
+     Cancel(3);
+}
+
+void cLiveSubtitle::Receive(uchar *Data, int Length)
+{
+  if (Running()) {
+     int p = ringBuffer->Put(Data, Length);
+     if (p != Length && Running())
+        ringBuffer->ReportOverflow(Length - p);
+     }
+}
+
+void cLiveSubtitle::Action(void)
+{
+  while (Running()) {
+        int Count;
+        uchar *b = ringBuffer->Get(Count);
+        if (b) {
+           Count = remux->Put(b, Count);
+           if (Count)
+              ringBuffer->Del(Count);
+           }
+        b = remux->Get(Count);
+        if (b) {
+           Count = cDevice::PrimaryDevice()->PlaySubtitle(b, Count);
+           remux->Del(Count);
+           }
+        }
+}
 
 // --- cPesAssembler ---------------------------------------------------------
 
@@ -155,6 +225,10 @@ cDevice::cDevice(void)
 
   SetVideoFormat(Setup.VideoFormat);
 
+  LNBstate = -1;
+  LNBnr = Setup.CardUsesLNBnr[cardIndex];
+  LNBsource = NULL;
+
   mute = false;
   volume = Setup.CurrentVolume;
 
@@ -172,6 +246,10 @@ cDevice::cDevice(void)
   ClrAvailableTracks();
   currentAudioTrack = ttNone;
   currentAudioTrackMissingCount = 0;
+  currentSubtitleTrack = ttNone;
+  liveSubtitle = NULL;
+  dvbSubtitleConverter = NULL;
+  autoSelectPreferredSubtitleLanguage = true;
 
   for (int i = 0; i < MAXRECEIVERS; i++)
       receiver[i] = NULL;
@@ -186,6 +264,8 @@ cDevice::~cDevice()
 {
   Detach(player);
   DetachAllReceivers();
+  delete liveSubtitle;
+  delete dvbSubtitleConverter;
   delete nitFilter;
   delete sdtFilter;
   delete patFilter;
@@ -216,6 +296,14 @@ void cDevice::SetUseDevice(int n)
      useDevice |= (1 << n);
 }
 
+void cDevice::SetLNBnr(void)
+{
+  for (int i = 0; i < numDevices; i++) {
+      device[i]->LNBnr = Setup.CardUsesLNBnr[i];
+      isyslog("LNB-sharing: setting device %d to use LNB %d", i, device[i]->LNBnr);
+      }
+}
+
 int cDevice::NextCardIndex(int n)
 {
   if (n > 0) {
@@ -251,6 +339,7 @@ bool cDevice::SetPrimaryDevice(int n)
      primaryDevice = device[n];
      primaryDevice->MakePrimaryDevice(true);
      primaryDevice->SetVideoFormat(Setup.VideoFormat);
+     primaryDevice->SetVolumeDevice(Setup.CurrentVolume);
      return true;
      }
   esyslog("ERROR: invalid primary device number: %d", n + 1);
@@ -275,6 +364,96 @@ cDevice *cDevice::ActualDevice(void)
   return d;
 }
 
+cDevice *cDevice::GetBadDevice(const cChannel *Channel)
+{
+  if (!cSource::IsSat(Channel->Source())) return NULL;
+  if (Setup.DiSEqC) {
+     cDiseqc *diseqc;
+     diseqc = Diseqcs.Get(Channel->Source(), Channel->Frequency(), Channel->Polarization());
+
+     for (int i = 0; i < numDevices; i++) {
+         if (this != device[i] && device[i]->GetLNBnr() == LNBnr && device[i]->GetLNBsource() != (int*) diseqc) {
+            if (Setup.VerboseLNBlog) {
+               isyslog("LNB %d: Device check for channel %d on device %d. LNB or DiSEq conflict with device %d", LNBnr, Channel->Number(), this->DeviceNumber(), i);
+               }
+            return device[i];
+            }
+         }
+     if (Setup.VerboseLNBlog) {
+        isyslog("LNB %d: Device check for for channel %d on device %d. OK", LNBnr, Channel->Number(), this->DeviceNumber());
+        }
+  } else {
+     char requiredState;
+     if (Channel->Frequency() >= Setup.LnbSLOF) {
+        requiredState = 1 ;
+     } else {
+        requiredState = 0;
+        }
+     if (Channel->Polarization() == 'v' || Channel->Polarization() == 'V') requiredState += 2;
+
+     for (int i = 0; i < numDevices; i++) {
+         if (this != device[i] && device[i]->GetLNBnr() == LNBnr && device[i]->GetLNBconf() != requiredState) {
+            if (Setup.VerboseLNBlog) {
+               isyslog("LNB %d: Device check for channel %d, LNBstate %d on device %d, current LNBstate %d. Conflict with device %d, LNBstate %d", LNBnr, Channel->Number(), requiredState, this->DeviceNumber(), LNBstate, i, device[i]->GetLNBconf());
+               }
+            return device[i];
+            }
+         }
+     if (Setup.VerboseLNBlog) {
+        isyslog("LNB %d: Device check for channel %d, LNBstate %d on device %d, current LNBstate %d. No other devices affected", LNBnr, Channel->Number(), requiredState, this->DeviceNumber(), LNBstate);
+        }
+     }
+  return NULL;
+}
+
+int cDevice::GetMaxBadPriority(const cChannel *Channel)
+{
+  if (!cSource::IsSat(Channel->Source())) return -2;
+  bool PrimaryIsBad = false;
+  int maxBadPriority = -2;
+  if (Setup.DiSEqC) {
+     cDiseqc *diseqc;
+     diseqc = Diseqcs.Get(Channel->Source(), Channel->Frequency(), Channel->Polarization());
+
+     for (int i = 0; i < numDevices; i++) {
+         if (this != device[i] && device[i]->GetLNBnr() == LNBnr && device[i]->GetLNBsource() != (int*) diseqc) {
+            if (device[i]->Receiving() && device[i]->Priority() > maxBadPriority) {
+               maxBadPriority = device[i]->Priority();
+               }
+            if (device[i]->IsPrimaryDevice()) {
+               PrimaryIsBad = true;
+               }
+            }
+         }
+  } else {
+     char requiredState;
+     if (Channel->Frequency() >= Setup.LnbSLOF) {
+        requiredState = 1 ;
+     } else {
+        requiredState = 0;
+        }
+     if (Channel->Polarization() == 'v' || Channel->Polarization() == 'V') requiredState += 2;
+
+     for (int i = 0; i < numDevices; i++) {
+         if (this != device[i] && device[i]->GetLNBnr() == LNBnr && device[i]->GetLNBconf() != requiredState) {
+            if (device[i]->Receiving() && device[i]->Priority() > maxBadPriority) {
+               maxBadPriority = device[i]->Priority();
+               }
+            if (device[i]->IsPrimaryDevice()) {
+               PrimaryIsBad = true;
+               }
+            }
+         }
+     }
+  if (PrimaryIsBad && maxBadPriority == -2) {
+     maxBadPriority = -1;
+     }
+  if (Setup.VerboseLNBlog) { 
+     isyslog("LNB %d: Request for channel %d on device %d. MaxBadPriority is %d", LNBnr, Channel->Number(), this->DeviceNumber(), maxBadPriority);
+     }
+  return maxBadPriority;
+}
+
 cDevice *cDevice::GetDevice(int Index)
 {
   return (0 <= Index && Index < numDevices) ? device[Index] : NULL;
@@ -307,6 +486,8 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
   cCamSlot *s = NULL;
 
   uint32_t Impact = 0xFFFFFFFF; // we're looking for a device with the least impact
+  int badPriority;
+  uint imp2;
   for (int j = 0; j < NumCamSlots || !NumUsableSlots; j++) {
       if (NumUsableSlots && SlotPriority[j] > MAXPRIORITY)
          continue; // there is no CAM available in this slot
@@ -337,7 +518,26 @@ cDevice *cDevice::GetDevice(const cChannel *Channel, int Priority, bool LiveView
              imp <<= 1; imp |= NumUsableSlots ? 0 : device[i]->HasCi();                                              // avoid cards with Common Interface for FTA channels
              imp <<= 1; imp |= device[i]->HasDecoder();                                                              // avoid full featured cards
              imp <<= 1; imp |= NumUsableSlots ? !ChannelCamRelations.CamDecrypt(Channel->GetChannelID(), j + 1) : 0; // prefer CAMs that are known to decrypt this channel
-             if (imp < Impact) {
+       badPriority = device[i]->GetMaxBadPriority(Channel);
+       if (badPriority >= Priority || (badPriority == -1 && Priority < Setup.PrimaryLimit)) {
+            // channel is not available for the requested prioity
+            imp = 0xFFFFFFFF;
+         } else {
+            switch (badPriority) {
+               case -2: // not affected by LNB-sharing
+                  imp2 = 0;
+                  break;
+               case -1: // the primary device would need a channel switch
+                  imp += 1 <<  17;
+                  imp2 = 0xFFFFFFFF | 1 << 17;
+                  break;
+               default: // a device receiving with lower priority would need to be stopped
+                  imp += badPriority << 8;
+                  imp2 = 0xFFFFFFFF | badPriority << 8;
+                  break;
+               }
+            }
+         if (imp < Impact && imp2 < Impact) {
                 // This device has less impact than any previous one, so we take it.
                 Impact = imp;
                 d = device[i];
@@ -576,6 +776,11 @@ int cDevice::OpenFilter(u_short Pid, u_char Tid, u_char Mask)
   return -1;
 }
 
+void cDevice::CloseFilter(int Handle)
+{
+  close(Handle);
+}
+
 void cDevice::AttachFilter(cFilter *Filter)
 {
   if (sectionHandler)
@@ -601,7 +806,7 @@ bool cDevice::ProvidesTransponder(const cChannel *Channel) const
 bool cDevice::ProvidesTransponderExclusively(const cChannel *Channel) const
 {
   for (int i = 0; i < numDevices; i++) {
-      if (device[i] && device[i] != this && device[i]->ProvidesTransponder(Channel))
+      if (device[i] && device[i] != this && device[i]->ProvidesTransponder(Channel) && device[i]->GetLNBnr() != LNBnr)
          return false;
       }
   return true;
@@ -623,6 +828,22 @@ bool cDevice::MaySwitchTransponder(void)
 }
 
 bool cDevice::SwitchChannel(const cChannel *Channel, bool LiveView)
+{
+  cDevice *tmpDevice;
+  if (this->GetMaxBadPriority(Channel) >= 0) {
+     Skins.Message(mtInfo, tr("Channel locked by LNB!"));
+     return false;
+     }
+  while ((tmpDevice = GetBadDevice(Channel)) != NULL) {
+        if (tmpDevice->IsPrimaryDevice() && LiveView)
+           tmpDevice->SwitchChannelForced(Channel, true);
+        else
+           tmpDevice->SwitchChannelForced(Channel, false);
+        }
+  return SwitchChannelForced(Channel, LiveView);
+}
+
+bool cDevice::SwitchChannelForced(const cChannel *Channel, bool LiveView)
 {
   if (LiveView) {
      isyslog("switching to channel %d", Channel->Number());
@@ -653,7 +874,7 @@ bool cDevice::SwitchChannel(int Direction)
      cChannel *channel;
      while ((channel = Channels.GetByNumber(n, Direction)) != NULL) {
            // try only channels which are currently available
-           if (GetDevice(channel, 0, true))
+           if (PrimaryDevice()->GetMaxBadPriority(channel) < 0 && (GetDevice(channel, 0, true)))
               break;
            n = channel->Number() + Direction;
            }
@@ -674,8 +895,11 @@ bool cDevice::SwitchChannel(int Direction)
 
 eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 {
-  if (LiveView)
+  if (LiveView) {
      StopReplay();
+     DELETENULL(liveSubtitle);
+     DELETENULL(dvbSubtitleConverter);
+     }
 
   cDevice *Device = (LiveView && IsPrimaryDevice()) ? GetDevice(Channel, 0, LiveView) : this;
 
@@ -683,11 +907,33 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
 
   eSetChannelResult Result = scrOk;
 
+  char requiredState;
+  if (Channel->Frequency() >= Setup.LnbSLOF) {
+     requiredState = 1;
+  } else {
+     requiredState = 0;
+     }
+  if (Channel->Polarization() == 'v' || Channel->Polarization() == 'V') requiredState += 2;
+  if (Setup.VerboseLNBlog) {
+     isyslog("LNB %d: Switching device %d to channel %d", LNBnr, this->DeviceNumber(), Channel->Number());
+     }
+
   // If this DVB card can't receive this channel, let's see if we can
   // use the card that actually can receive it and transfer data from there:
 
   if (NeedsTransferMode) {
      if (Device && CanReplay()) {
+        if (Device->GetLNBnr() == LNBnr) {
+           if (LNBstate != requiredState || (Setup.DiSEqC && LNBsource != (int*) Diseqcs.Get(Channel->Source(), Channel->Frequency(), Channel->Polarization())) ) {
+              if (IsPrimaryDevice()) {
+                 SetChannelDevice(Channel, true);
+              } else {
+                 SetChannelDevice(Channel, false);
+                 }
+              LNBstate = requiredState;
+              LNBsource = (int*) Diseqcs.Get(Channel->Source(), Channel->Frequency(), Channel->Polarization());
+              }
+           }
         cStatus::MsgChannelSwitch(this, 0); // only report status if we are actually going to switch the channel
         if (Device->SetChannel(Channel, false) == scrOk) // calling SetChannel() directly, not SwitchChannel()!
            cControl::Launch(new cTransferControl(Device, Channel->GetChannelID(), Channel->Vpid(), Channel->Apids(), Channel->Dpids(), Channel->Spids()));
@@ -705,6 +951,8 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
         sectionHandler->SetStatus(false);
         sectionHandler->SetChannel(NULL);
         }
+     LNBstate = requiredState;
+     LNBsource = (int*) Diseqcs.Get(Channel->Source(), Channel->Frequency(), Channel->Polarization());
      // Tell the camSlot about the channel switch and add all PIDs of this
      // channel to it, for possible later decryption:
      if (camSlot)
@@ -735,8 +983,11 @@ eSetChannelResult cDevice::SetChannel(const cChannel *Channel, bool LiveView)
            for (int i = 0; i < MAXDPIDS; i++)
                SetAvailableTrack(ttDolby, i, Channel->Dpid(i), Channel->Dlang(i));
            }
+        for (int i = 0; i < MAXSPIDS; i++)
+            SetAvailableTrack(ttSubtitle, i, Channel->Spid(i), Channel->Slang(i));
         if (!NeedsTransferMode)
            EnsureAudioTrack(true);
+        EnsureSubtitleTrack();
         }
      cStatus::MsgChannelSwitch(this, Channel->Number()); // only report status if channel switch successfull
      }
@@ -848,6 +1099,7 @@ void cDevice::ClrAvailableTracks(bool DescriptionsOnly, bool IdsOnly)
      SetAudioChannel(0); // fall back to stereo
      currentAudioTrackMissingCount = 0;
      currentAudioTrack = ttNone;
+     currentSubtitleTrack = ttNone;
      }
 }
 
@@ -855,18 +1107,23 @@ bool cDevice::SetAvailableTrack(eTrackType Type, int Index, uint16_t Id, const c
 {
   eTrackType t = eTrackType(Type + Index);
   if (Type == ttAudio && IS_AUDIO_TRACK(t) ||
-      Type == ttDolby && IS_DOLBY_TRACK(t)) {
+      Type == ttDolby && IS_DOLBY_TRACK(t) ||
+      Type == ttSubtitle && IS_SUBTITLE_TRACK(t)) {
      if (Language)
         strn0cpy(availableTracks[t].language, Language, sizeof(availableTracks[t].language));
      if (Description)
         Utf8Strn0Cpy(availableTracks[t].description, Description, sizeof(availableTracks[t].description));
      if (Id) {
         availableTracks[t].id = Id; // setting 'id' last to avoid the need for extensive locking
-        int numAudioTracks = NumAudioTracks();
-        if (!availableTracks[currentAudioTrack].id && numAudioTracks && currentAudioTrackMissingCount++ > numAudioTracks * 10)
-           EnsureAudioTrack();
-        else if (t == currentAudioTrack)
-           currentAudioTrackMissingCount = 0;
+        if (Type == ttAudio || Type == ttDolby) {
+           int numAudioTracks = NumAudioTracks();
+           if (!availableTracks[currentAudioTrack].id && numAudioTracks && currentAudioTrackMissingCount++ > numAudioTracks * 10)
+              EnsureAudioTrack();
+           else if (t == currentAudioTrack)
+              currentAudioTrackMissingCount = 0;
+           }
+        else if (Type == ttSubtitle && autoSelectPreferredSubtitleLanguage)
+           EnsureSubtitleTrack();
         }
      return true;
      }
@@ -880,19 +1137,29 @@ const tTrackId *cDevice::GetTrack(eTrackType Type)
   return (ttNone < Type && Type < ttMaxTrackTypes) ? &availableTracks[Type] : NULL;
 }
 
-int cDevice::NumAudioTracks(void) const
+int cDevice::NumTracks(eTrackType FirstTrack, eTrackType LastTrack) const
 {
   int n = 0;
-  for (int i = ttAudioFirst; i <= ttDolbyLast; i++) {
+  for (int i = FirstTrack; i <= LastTrack; i++) {
       if (availableTracks[i].id)
          n++;
       }
   return n;
 }
 
+int cDevice::NumAudioTracks(void) const
+{
+  return NumTracks(ttAudioFirst, ttDolbyLast);
+}
+
+int cDevice::NumSubtitleTracks(void) const
+{
+  return NumTracks(ttSubtitleFirst, ttSubtitleLast);
+}
+
 bool cDevice::SetCurrentAudioTrack(eTrackType Type)
 {
-  if (ttNone < Type && Type < ttDolbyLast) {
+  if (ttNone < Type && Type <= ttDolbyLast) {
      cMutexLock MutexLock(&mutexCurrentAudioTrack);
      if (IS_DOLBY_TRACK(Type))
         SetDigitalAudioDevice(true);
@@ -908,6 +1175,30 @@ bool cDevice::SetCurrentAudioTrack(eTrackType Type)
   return false;
 }
 
+bool cDevice::SetCurrentSubtitleTrack(eTrackType Type, bool Manual)
+{
+  if (Type == ttNone || IS_SUBTITLE_TRACK(Type)) {
+     currentSubtitleTrack = Type;
+     autoSelectPreferredSubtitleLanguage = !Manual;
+     if (dvbSubtitleConverter)
+        dvbSubtitleConverter->Reset();
+     if (Type == ttNone && dvbSubtitleConverter) {
+        cMutexLock MutexLock(&mutexCurrentSubtitleTrack);
+        DELETENULL(dvbSubtitleConverter);
+        }
+     DELETENULL(liveSubtitle);
+     if (currentSubtitleTrack != ttNone && !Replaying() && !Transferring()) {
+        const tTrackId *TrackId = GetTrack(currentSubtitleTrack);
+        if (TrackId && TrackId->id) {
+           liveSubtitle = new cLiveSubtitle(TrackId->id);
+           AttachReceiver(liveSubtitle);
+           }
+        }
+     return true;
+     }
+  return false;
+}
+
 void cDevice::EnsureAudioTrack(bool Force)
 {
   if (Force || !availableTracks[currentAudioTrack].id) {
@@ -939,6 +1230,25 @@ void cDevice::EnsureAudioTrack(bool Force)
      }
 }
 
+void cDevice::EnsureSubtitleTrack(void)
+{
+  if (Setup.DisplaySubtitles) {
+     eTrackType PreferredTrack = ttSubtitleFirst;
+     int LanguagePreference = -1;
+     for (int i = ttSubtitleFirst; i <= ttSubtitleLast; i++) {
+         const tTrackId *TrackId = GetTrack(eTrackType(i));
+         if (TrackId && TrackId->id && I18nIsPreferredLanguage(Setup.SubtitleLanguages, TrackId->language, LanguagePreference))
+            PreferredTrack = eTrackType(i);
+         }
+     // Make sure we're set to an available subtitle track:
+     const tTrackId *Track = GetTrack(GetCurrentSubtitleTrack());
+     if (!Track || !Track->id || PreferredTrack != GetCurrentSubtitleTrack())
+        SetCurrentSubtitleTrack(PreferredTrack);
+     }
+  else
+     SetCurrentSubtitleTrack(ttNone);
+}
+
 bool cDevice::CanReplay(void) const
 {
   return HasDecoder();
@@ -961,6 +1271,8 @@ void cDevice::TrickSpeed(int Speed)
 void cDevice::Clear(void)
 {
   Audios.ClearAudio();
+  if (dvbSubtitleConverter)
+     dvbSubtitleConverter->Reset();
 }
 
 void cDevice::Play(void)
@@ -1016,6 +1328,9 @@ void cDevice::Detach(cPlayer *Player)
      player = NULL; // avoids recursive calls to Detach()
      p->Activate(false);
      p->device = NULL;
+     cMutexLock MutexLock(&mutexCurrentSubtitleTrack);
+     delete dvbSubtitleConverter;
+     dvbSubtitleConverter = NULL;
      SetPlayMode(pmNone);
      SetVideoDisplayFormat(eVideoDisplayFormat(Setup.VideoDisplayFormat));
      Audios.ClearAudio();
@@ -1051,6 +1366,13 @@ int cDevice::PlayAudio(const uchar *Data, int Length, uchar Id)
   return -1;
 }
 
+int cDevice::PlaySubtitle(const uchar *Data, int Length)
+{
+  if (!dvbSubtitleConverter)
+     dvbSubtitleConverter = new cDvbSubtitleConverter;
+  return dvbSubtitleConverter->Convert(Data, Length);
+}
+
 int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
 {
   cMutexLock MutexLock(&mutexCurrentAudioTrack);
@@ -1076,17 +1398,15 @@ int cDevice::PlayPesPacket(const uchar *Data, int Length, bool VideoOnly)
                break;
           case 0xBD: { // private stream 1
                int PayloadOffset = Data[8] + 9;
+
+               // Compatibility mode for old subtitles plugin:
+               if ((Data[PayloadOffset - 3] & 0x81) == 1 && Data[PayloadOffset - 2] == 0x81)
+                  PayloadOffset--;
+
                uchar SubStreamId = Data[PayloadOffset];
                uchar SubStreamType = SubStreamId & 0xF0;
                uchar SubStreamIndex = SubStreamId & 0x1F;
 
-               // Compatibility mode for subtitles plugin
-               if (((Data[PayloadOffset - 3] & 0x81) == 1) && (Data[PayloadOffset - 2] == 0x81)) {
-                  PayloadOffset = Data[8] + 9 - 1;
-                  SubStreamId = Data[PayloadOffset];
-                  SubStreamType = SubStreamId & 0xF0;
-                  SubStreamIndex = SubStreamId & 0x1F;
-                  }
                // Compatibility mode for old VDR recordings, where 0xBD was only AC3:
 pre_1_3_19_PrivateStreamDeteced:
                if (pre_1_3_19_PrivateStream) {
@@ -1097,6 +1417,9 @@ pre_1_3_19_PrivateStreamDeteced:
                switch (SubStreamType) {
                  case 0x20: // SPU
                  case 0x30: // SPU
+                      SetAvailableTrack(ttSubtitle, SubStreamIndex, SubStreamId);
+                      if (!VideoOnly && currentSubtitleTrack != ttNone && SubStreamId == availableTracks[currentSubtitleTrack].id)
+                         w = PlaySubtitle(Start, d);
                       break;
                  case 0x80: // AC3 & DTS
                       if (Setup.UseDolbyDigital) {
@@ -1146,6 +1469,8 @@ int cDevice::PlayPes(const uchar *Data, int Length, bool VideoOnly)
 {
   if (!Data) {
      pesAssembler->Reset();
+     if (dvbSubtitleConverter)
+        dvbSubtitleConverter->Reset();
      return 0;
      }
   int Result = 0;
