@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 1.46 2007/04/28 14:55:22 kls Exp $
+ * $Id: dvbplayer.c 1.47 2007/10/13 12:20:58 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -14,7 +14,6 @@
 #include "ringbuffer.h"
 #include "thread.h"
 #include "tools.h"
-#include "rcontroller.h"
 #include "vdrttxtsubshooks.h"
 
 // --- cBackTrace ------------------------------------------------------------
@@ -322,15 +321,8 @@ static void StripExtendedPackets(uchar *b, int Length)
          int l = b[i + 4] * 256 + b[i + 5] + 6;
          switch (c) {
            case 0xBD: // dolby
-                if (RecordingPatch::RecordingController.isExtendedPacket(b + i , l)) {
-                   RecordingPatch::RecordingController.Receive( b + i, l );
-                   // continue with deleting the data - otherwise it disturbs DVB replay
-                   int n = l;
-                   for (int j = i; j < Length && n--; j++)
-                       b[j] = 0x00;
-                   }
                 // EBU Teletext data, ETSI EN 300 472
-                else if (b[i + 8] == 0x24 && b[i + 45] >= 0x10 && b[i + 45] < 0x20) {
+                if (b[i + 8] == 0x24 && b[i + 45] >= 0x10 && b[i + 45] < 0x20) {
                    cVDRTtxtsubsHookListener::Hook()->PlayerTeletextData(&b[i], l);
                    // continue with deleting the data - otherwise it disturbs DVB replay
                    int n = l;
@@ -631,13 +623,19 @@ void cDvbPlayer::Action(void)
                        uchar FileNumber;
                        int FileOffset;
                        bool TimeShiftMode = index->IsStillRecording();
-                       int Index = index->GetNextIFrame(readIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length, TimeShiftMode);
+                       int Index = -1;
+                       if (DeviceHasIBPTrickSpeed() && playDir == pdForward) {
+                          if (index->Get(readIndex + 1, &FileNumber, &FileOffset, NULL, &Length))
+                             Index = readIndex + 1;
+                          }
+                       else
+                          Index = index->GetNextIFrame(readIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length, TimeShiftMode);
                        if (Index >= 0) {
                           if (!NextFile(FileNumber, FileOffset)) {
                              readIndex = Index;
                              continue;
                              }
-                          Length += IPACKS; // fixIFrame needs next video packet
+			  Length += IPACKS; // fixIFrame needs next video packet
                           }
                        else {
                           if (!TimeShiftMode && playDir == pdForward) {
@@ -682,8 +680,8 @@ void cDvbPlayer::Action(void)
                     }
                  int r = nonBlockingFileReader->Read(replayFile, b, Length);
                  if (r > 0) {
-                    if (playMode == pmFast || (playMode == pmSlow && playDir == pdBackward))
-                       fixIFrame(b, r, Length - IPACKS);
+		    if (playMode == pmFast || (playMode == pmSlow && playDir == pdBackward))
+		       fixIFrame(b, r, Length - IPACKS);
                     WaitingForData = false;
                     readFrame = new cFrame(b, -r, ftUnknown, readIndex); // hands over b to the ringBuffer
                     b = NULL;
@@ -927,7 +925,7 @@ void cDvbPlayer::Goto(int Index, bool Still)
         uchar b[MAXFRAMESIZE + 4 + 5 + 4];
         int r = ReadFrame(replayFile, b, Length, sizeof(b));
         if (r > 0) {
-           fixIFrame(b, r, Length - IPACKS);
+	   fixIFrame(b, r, Length - IPACKS);
            if (playMode == pmPause)
               DevicePlay();
            // append sequence end code to get the image shown immediately with softdevices
