@@ -11,7 +11,7 @@
  * The cRepacker family's code was originally written by Reinhard Nissl <rnissl@gmx.de>,
  * and adapted to the VDR coding style by Klaus.Schmidinger@cadsoft.de.
  *
- * $Id: remux.c 1.58 2007/02/24 16:36:10 kls Exp $
+ * $Id: remux.c 1.59 2007/09/22 12:08:22 kls Exp $
  */
 
 #include "remux.h"
@@ -1413,9 +1413,6 @@ int cDolbyRepacker::BreakAt(const uchar *Data, int Count)
 //pts_dts flags
 #define PTS_ONLY         0x80
 
-#define PES_EXTENSION    0x01
-#define PES_EXTENSION2   0x01
-
 #define TS_SIZE        188
 #define PID_MASK_HI    0x1F
 #define CONT_CNT_MASK  0x0F
@@ -1464,8 +1461,6 @@ private:
   int ccErrors;
   int ccCounter;
   cRepacker *repacker;
-  uint8_t dataIdentifier;
-  static uint8_t eHeadr[];
   static uint8_t headr[];
   void store(uint8_t *Data, int Count);
   void reset_ipack(void);
@@ -1473,18 +1468,16 @@ private:
   void write_ipack(const uint8_t *Data, int Count);
   void instant_repack(const uint8_t *Buf, int Count);
 public:
-  cTS2PES(int Pid, cRingBufferLinear *ResultBuffer, int Size, uint8_t RewriteCid = 0x00, uint8_t SubStreamId = 0x00, cRepacker *Repacker = NULL, uint8_t DataIdentifier = 0x00);
-
+  cTS2PES(int Pid, cRingBufferLinear *ResultBuffer, int Size, uint8_t RewriteCid = 0x00, uint8_t SubStreamId = 0x00, cRepacker *Repacker = NULL);
   ~cTS2PES();
   int Pid(void) { return pid; }
   void ts_to_pes(const uint8_t *Buf); // don't need count (=188)
   void Clear(void);
   };
 
-uint8_t cTS2PES::eHeadr[] = { 0x01, 0x81 }; // extension header
 uint8_t cTS2PES::headr[] = { 0x00, 0x00, 0x01 };
 
-cTS2PES::cTS2PES(int Pid, cRingBufferLinear *ResultBuffer, int Size, uint8_t RewriteCid, uint8_t SubStreamId, cRepacker *Repacker, uint8_t DataIdentifier)
+cTS2PES::cTS2PES(int Pid, cRingBufferLinear *ResultBuffer, int Size, uint8_t RewriteCid, uint8_t SubStreamId, cRepacker *Repacker)
 {
   pid = Pid;
   resultBuffer = ResultBuffer;
@@ -1501,7 +1494,6 @@ cTS2PES::cTS2PES(int Pid, cRingBufferLinear *ResultBuffer, int Size, uint8_t Rew
   tsErrors = 0;
   ccErrors = 0;
   ccCounter = -1;
-  dataIdentifier = DataIdentifier;
 
   if (!(buf = MALLOC(uint8_t, size)))
      esyslog("Not enough memory for ts_transform");
@@ -1559,21 +1551,17 @@ void cTS2PES::send_ipack(void)
 
   switch (mpeg) {
     case 2:
-         if ( dataIdentifier == 0 ) {
             buf[6] = 0x80;
             buf[7] = 0x00;
             buf[8] = 0x00;
             count = 9;
-            }
-         else {
-            buf[6] = 0x80;
-            buf[7] = 0x01; // pes_extension_flag == 1
-            buf[8] = 0x03; // pes_header_data_length == 3
-            buf[9] = 0x01; // pes_extension_flag_2=1
-            buf[10]= 0x81; // marker_bit=1, pes_extension_data_length=1 
-            buf[11] = dataIdentifier;
-            count = 12;
-            }
+            if (!repacker && subStreamId) {
+               buf[9] = subStreamId;
+               buf[10] = 1;
+               buf[11] = 0;
+               buf[12] = 1;
+               count = 13;
+               }
             break;
     case 1:
             buf[6] = 0x0F;
@@ -1736,21 +1724,12 @@ void cTS2PES::instant_repack(const uint8_t *Buf, int Count)
           case 7:
                   if (!done && (mpeg == 2 || mpeg1_required > 7)) {
                      flag2 = Buf[c++];
-                     if ( dataIdentifier && (flag2 & PES_EXTENSION) ) {
-                        esyslog("Error: cannot add extension to pes packet. Disabling.");
-                        dataIdentifier = 0;
-                        }
-                     else {
-                        flag2 |= PES_EXTENSION;
-                        }
                      found++;
                      }
                   break;
           case 8:
                   if (!done && (mpeg == 2 || mpeg1_required > 7)) {
                      hlength = Buf[c++];
-                     if ( dataIdentifier )
-                        hlength += 3;
                      found++;
                      if (mpeg == 1 && hlength != 0x0F && (hlength & 0xF0) != 0x20 && (hlength & 0xF0) != 0x30)
                         found = 0; // invalid MPEG1 header
@@ -1794,17 +1773,16 @@ void cTS2PES::instant_repack(const uint8_t *Buf, int Count)
                   return;
                }
 
-            // Write header one byte at a time
-            // Remove from hlength size of our header (3)
-            if ( dataIdentifier ) {
-               while (c < Count && (found < (hlength + 9-3)) && found < plength+6) {
+            if (!repacker && subStreamId) {
+               while (c < Count && found < (hlength + 9) && found < plength + 6) {
                      write_ipack(Buf + c, 1);
                      c++;
                      found++;
                      }
-               if (found == (hlength+9-3)) {
-                  write_ipack(eHeadr, 2); 
-                  write_ipack(&dataIdentifier, 1);
+               if (found == (hlength + 9)) {
+                  uchar sbuf[] = { 0x01, 0x00, 0x00 };
+                  write_ipack(&subStreamId, 1);
+                  write_ipack(sbuf, 3);
                   }
                }
 
@@ -1898,7 +1876,7 @@ void cTS2PES::ts_to_pes(const uint8_t *Buf) // don't need count (=188)
 cRemux::cRemux(int VPid, const int *APids, const int *DPids, const int *SPids, bool ExitOnFailure)
 {
   exitOnFailure = ExitOnFailure;
-  isRadio = VPid == 0 || VPid == 1 || VPid == 0x1FFF;
+  noVideo = VPid == 0 || VPid == 1 || VPid == 0x1FFF;
   numUPTerrors = 0;
   synced = false;
   skipped = 0;
@@ -1933,7 +1911,7 @@ cRemux::cRemux(int VPid, const int *APids, const int *DPids, const int *SPids, b
   if (SPids) {
      int n = 0;
      while (*SPids && numTracks < MAXTRACKS && n < MAXSPIDS)
-           ts2pes[numTracks++] = new cTS2PES(*SPids++, resultBuffer, IPACKS, 0x00, 0x00, NULL, 0x28 + n++);
+           ts2pes[numTracks++] = new cTS2PES(*SPids++, resultBuffer, IPACKS, 0x00, 0x20 + n++);
      }
 }
 
@@ -2120,7 +2098,7 @@ uchar *cRemux::Get(int &Count, uchar *PictureType)
                l = GetPacketLength(data, resultCount, i);
                if (l < 0)
                   return resultData;
-               if (isRadio) {
+               if (noVideo) {
                   if (!synced) {
                      if (PictureType)
                         *PictureType = I_FRAME;
